@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -242,14 +242,21 @@ typedef struct {
   /**
    * @brief The genre
    */
-  unsigned char genre;
+  char *genre;
 
   /*
    * @brief tags version (for mp3): 1 or 2 or 1 & 2
    */
   int tags_version;
+
+  int set_original_tags;
 } splt_tags;
 
+typedef struct {
+  splt_tags tags;
+  void *all_original_tags;
+} splt_original_tags;
+
 /**
  * @brief Definition of a splitpoint
  *
@@ -286,54 +293,67 @@ struct splt_ssplit {
 /* Structure for the split        */
 
 /**
- * @brief must be documented
+ * @brief The data structure for the progress bar
+
+  The meanings of the values are:
+
+  float  = fraction in percent
+  char * = string (name on the progress bar)
+
  */
 typedef struct splt_progres {
-  //maximum number of characters for the filename(without the
-  //extension) when displaying the progress
-  //warning; default is 40
+  /*!maximum number of characters for the filename
+  
+    (without the extension) when displaying the 
+    progress warning; default is 40 */
   int progress_text_max_char;
-  //filename that we are currently splitting
+  //! Name of the file  we are currently splitting
   char filename_shorted[512];
-  //the current percent of the progress
+  //!the current percent of the progress
   float percent_progress;
-  //the splitpoint that we are currently splitting
+  //!the splitpoint we are currently splitting
   int current_split;
-  //the maximum number of splits
+  //!the maximum number of splits
   int max_splits;
-  //the progress type
-  //can be:
-  //SPLT_PROGRESS_PREPARE
-  //SPLT_PROGRESS_CREATE
-  //SPLT_PROGRESS_SEARCH_SYNC
-  //SPLT_PROGRESS_SCAN_SILENCE
+  /*!the progress bar type
+
+    can be:
+    - SPLT_PROGRESS_PREPARE
+    - SPLT_PROGRESS_CREATE
+    - SPLT_PROGRESS_SEARCH_SYNC
+    - SPLT_PROGRESS_SCAN_SILENCE */
   int progress_type;
-  //infos for the silence split
+  //!infos for the silence split
   int silence_found_tracks;
+  //! The silence leven in dB
   float silence_db_level;
-  //use this variable as you wish
-  //this variable will not be modified by the library
-  //but is 0 at the start
+  /*! use this variable as you wish
+  
+    this variable will not be modified by the library
+    but it will be 0 at the start
+  */
   int user_data;
-  //float = fraction of how much %
-  //char * = name on the progress bar
+  //! A pointer to the callback function
   void (*progress)(struct splt_progres*);
 } splt_progress;
 
-//proxy informations
+//!proxy information
 typedef struct {
-  //if we use proxy or not
+  //! True if we use a proxy
   int use_proxy;
+  //! The name of the proxy
   char hostname[256];
+  //! The port we use to access the procy
   int port;
-  //if we need authentification or not
+  //! True if we need authentification
   int authentification;
-  //the user and password needed for authentification
+  //! the username needed for authentification
   char user[256];
+  //! the password needed for authentification
   char password[256];
 } splt_proxy;
 
-//used with the 'put_message' function
+//!used with the 'put_message' function
 typedef enum {
   /**
    * Info message
@@ -345,40 +365,45 @@ typedef enum {
   SPLT_MESSAGE_DEBUG
 } splt_message_type;
 
+//! The data libmp3splt keeps all its internal data in
 typedef struct {
-  //total time of the song
+  //! total time of the song
   long total_time;
-  //the part of file that we are currently splitting
-  //1 if the first, 2 if the second
-  //the number of splitpoints
+  //!When splitting: the number of the last splitpoint we arrived at.
   int current_split;
-  //for the user feedback: the current split file number
+  //! for user feedback: the number of the file we are splitting
   int current_split_file_number;
-  //how many splits, this will be modified
-  //by check_splitpts_inf_song_length()
-  //to really see how many splitpoints we have
-  //look at real_splitnumber
+  /*! how many split points do we have?
+    
+    this is updated only by
+    check_splitpts_inf_song_length()
+    If this function has not been called
+    look at real_splitnumber instead.
+  */
   int splitnumber;
-  //how many splitpoints we have
+  //!how many splitpoints do we have?
   int real_splitnumber;
-  //put this function if you want that the library
-  //tells you when a file has been split
-  //the char* is the filename
+  /*! Pointer to the fallback function that is 
+      called when the library starts to split
+      a new file.
+      
+      the char* is the filename that is currently
+      being split.
+  */
   void (*file_split)(const char *,int);
-  //for the progress bar
+  //!All infos for the progress bar
   splt_progress *p_bar;
-  //callback for sending the silence level to the client
+  //!callback for sending the silence level to the client
   void (*get_silence_level)(long time, float level, void *user_data);
-  //user data set by the client for the 'get_silence_level' function
+  //!user data set by the client for the 'get_silence_level' function
   void *silence_level_client_data;
-  //sends a message to the main program to tell him what
-  //he is doing; the second parameter is the type of split
+  //!sends a message to the main program to tell what we are doing
   void (*put_message)(const char *, splt_message_type );
-  //structure in which we have all the splitpoints
+  //!structure in which we have all the splitpoints
   splt_point *points;
-  //how many tags we have
+  //!how many tags do we have?
   int real_tagsnumber;
-  //structure in which we have all the tags
+  //!structure in which we have all the tags
   splt_tags *tags;
   splt_tags tags_like_x;
 } splt_struct;
@@ -404,6 +429,10 @@ typedef enum {
    * Split with silence detection
    */
   SPLT_OPTION_SILENCE_MODE,
+  /**
+   * Split with trim silence detection
+   */
+  SPLT_OPTION_TRIM_SILENCE_MODE,
   /**
    * Split with error mode 
    * It is useful to split large file derivated from a concatenation of
@@ -428,13 +457,15 @@ typedef enum {
  * Values for the #SPLT_OPT_OUTPUT_FILENAMES option
  */
 typedef enum {
-  //output specified by the set_oformat
+  //!output specified by the set_oformat
   SPLT_OUTPUT_FORMAT,
-  //the default output
-  //it depends of the type of the split
+  //!the default output. depends of the type of the split.
   SPLT_OUTPUT_DEFAULT,
-  //we don't change anything, must put the filenames with
-  //the functions set_..
+  /*!we don't change anything
+    
+    Of course we use our internal logic to generate at 
+    least unique names in this case
+  */
   SPLT_OUTPUT_CUSTOM
 } splt_output_filenames_options;
 
@@ -477,7 +508,12 @@ typedef enum {
   /**
    * Does not put any tags
    */
-  SPLT_NO_TAGS
+  SPLT_NO_TAGS,
+  /**
+   * Sets tags from filename regex.
+   * See #mp3splt_set_input_filename_regex.
+   */
+  SPLT_TAGS_FROM_FILENAME_REGEX,
 } splt_tags_options;
 
 #define SPLT_ORIGINAL_TAGS_DEFAULT "%[@o,@N=1]"
@@ -499,96 +535,144 @@ typedef enum {
  * See #mp3splt_set_oformat
  */
 #define SPLT_DEFAULT_SILENCE_OUTPUT "@f_silence_@n"
+/**
+ * @brief Default output for the trim silence split.
+ * See #mp3splt_set_oformat
+ */
+#define SPLT_DEFAULT_TRIM_SILENCE_OUTPUT "@f_trimmed"
 
-//structure with all the options supplied to split the file
+//!structure with all the options supplied to split the file
 typedef struct {
-  //this can take the following values:
-  //SPLT_OPTION_NORMAL_MODE
-  //SPLT_OPTION_WRAP_MODE
-  //SPLT_OPTION_SILENCE_MODE
-  //SPLT_OPTION_ERROR_MODE
-  //SPLT_OPTION_TIME_MODE
+  /*! this field can take the following values:
+     - SPLT_OPTION_NORMAL_MODE
+     - SPLT_OPTION_WRAP_MODE
+     - SPLT_OPTION_SILENCE_MODE
+     - SPLT_OPTION_TRIM_SILENCE_MODE
+     - SPLT_OPTION_ERROR_MODE
+     - SPLT_OPTION_TIME_MODE
+  */
   splt_split_mode_options split_mode;
 
-  //might be:
-  //SPLT_TAGS_ORIGINAL_FILE - write tags from original file
-  //SPLT_NO_TAGS - does not write any tags
-  //SPLT_CURRENT_TAGS - tags issued from the cddb or cue for example
-  //or that we set manually with the functions
+  /*! might be:
+       - SPLT_TAGS_ORIGINAL_FILE - write tags from original file
+       - SPLT_NO_TAGS - does not write any tags
+       - SPLT_CURRENT_TAGS - tags issued from the cddb or cue for example
+       - or be set manually with the functions
+         splt_tags_options tags;
+  */
   splt_tags_options tags;
 
   int xing;
 
-  //defines the output filenames
+  //!defines the output filenames
   splt_output_filenames_options output_filenames;
 
-  //quiet mode: don't perform CRC check or other interaction with the user
+  //!quiet mode: don't perform CRC check or other interaction with the user
   int quiet_mode;
 
-  //Pretend to split the file, without real split: this option works in
-  //all modes except error mode and dewrap split.
+  /*!Pretend to split the file, without real split
+
+    this option works in all modes except error mode and dewrap split. 
+  */
   int pretend_to_split;
 
-  //frame mode (mp3 only). Process all frames, seeking split positions
-  //by counting frames and not with bitrate guessing.
+  /*! frame mode (mp3 only). 
+
+    Process all frames, seeking split positions by counting frames
+    instead of bitrate guessing.
+  */
   int option_frame_mode;
-  //the time of split when split_mode = OPTION_TIME_SPLIT
+  //!the time of split when split_mode = OPTION_TIME_SPLIT
   float split_time;
   long overlap_time;
-  //this option uses silence detection to auto-adjust splitpoints.
+  //!this option uses silence detection to auto-adjust splitpoints.
   int option_auto_adjust;
-  //input not seekable. enabling this allows you to split mp3 and ogg streams
-  //which can be read only one time and canât be seeked.
-  //WARNING!
-  //if you don't know what this means, set it to FALSE
+  /*! True means: input not seekable. 
+
+    enabling this allows you to split mp3 and ogg streams
+    which can be read only one time and canât be seeked.
+    WARNING!
+    if you don't know what this means you know why it is wise
+    to set this to FALSE.
+  */
   int option_input_not_seekable;
 
-  //If this option is SPLT_TRUE, we create directories from the output
-  //file names without parsing for illegal characters the output filenames.
-  //Otherwise, we parse for illegal characters the filenames and replace
-  //them with '_'. The tags are always checked for illegal characters when
-  //set into filenames.
+  /*! SPLT_TRUE= don't patse output filenames for illegal characters.
+
+    Otherwise, we parse for illegal characters the filenames and replace
+    them with '_'. The tags are always checked for illegal characters when
+    set into filenames.
+  */
   int create_dirs_from_filenames;
 
   //PARAMETERS---------------------------------------
   //PARAMETERS for option_auto_adjust and option_silence_mode:
-  //the sound level to be considered silence
-  //(it is a float number between -96 and 0. Default is -48 dB)
+  /*! the sound level to be considered silence
+ 
+   (is a float number between -96 and 0. Default is -48 dB)*/
   float parameter_threshold;
-  //the offset of cutpoint in silence
-  //Float number between -2 and 2 and allows you to adjust the offset
-  //of cutpoint in silence time.0 is the begin of silence, and 1 the
-  //end;default is 0.8. 
+  /*! the offset of cutpoint in silence
+
+    Float number between -2 and 2 and allows you to adjust the offset
+    of cutpoint in silence time.0 is the begin of silence, and 1 the
+    end;default is 0.8. 
+
+    Used for option_auto_adjust and option_silence_mode
+  */
   float parameter_offset;
 
   //PARAMETERS for option_silence_mode:
-  //the desired number of tracks
-  //(positive integer number of tracks to be split;by default all
-  //tracks are split)
+  /*! the desired number of tracks for option_silence_mode
+
+  (positive integer number of tracks to be split;by default all
+  tracks are split)
+  */
   int parameter_number_tracks;
-  //the minimum silence length in seconds
-  //(positive float of the minimum number of seconds to be considered
-  //a valid splitpoint)
+  /*! The minimum silence length for option_silence_mode [in seconds]
+    
+    A positive float of the minimum number of seconds to be considered
+    a valid splitpoint
+  */
   float parameter_minimum_length;
-  //allows you to remove the silence between split tracks
+
+  //!possible values are #splt_str_format
+  int artist_tag_format;
+  //!possible values are #splt_str_format
+  int album_tag_format;
+  //!possible values are #splt_str_format
+  int title_tag_format;
+  //!possible values are #splt_str_format
+  int comment_tag_format;
+
+  //!true=replace underscores with space
+  int replace_underscores_tag_format;
+
+  //!SPLT_TRUE=set the current filename to split from FILE tag - if found in the CUE file
+  int set_file_from_cue_if_file_tag_found;
+
+  //!true=remove the silence between split tracks
   int parameter_remove_silence;
 
   //PARAMETERS for option_auto_adjust:
-  //the gap value around splitpoint to search for silence
-  //(positive integer for the time to decode before and after
-  //splitpoint;default gap is 30 seconds)
+  /*! Auto-Adjust: the gap value around splitpoint to search for
+    silence 
+    
+    (positive integer for the time to decode before and after
+    splitpoint;default gap is 30 seconds)
+  */
   int parameter_gap;
 
-  //-if we set all the remaining tags (for which we have no tags structure
-  //like the tag number 'remaining_tags_like_x'
-  //- set to -1 to disable
+  /*! Copy all tags from tag number 'remaining_tags_like_x'
+    
+    set to -1 to disable
+  */
   int remaining_tags_like_x;
 
-  //if to auto increment the tracknumber of the tags
+  //!true=auto-increment the tracknumber of the tags
   int auto_increment_tracknumber_tags;
 
   /**
-   * if we enable the silence points log ('mp3splt.log')
+   * true=enable the silence points log ('mp3splt.log')
    */
   int enable_silence_log;
 
@@ -612,23 +696,21 @@ typedef struct {
 //internal structures
 typedef struct
 {
-  //if we have send the message frame mode enabled
+  //!Do we have send the message frame mode enabled?
   int frame_mode_enabled;
-  //if current_refresh_rate = refresh_rate, we call
-  //the progress callback
+  //!if current_refresh_rate = refresh_rate, we call the progress callback
   int current_refresh_rate;
-  //if set to SPLT_TRUE,
-  //then we don't send messages to clients
+  //! if set to SPLT_TRUE then we don't send messages to clients
   int messages_locked;
-  //if we currently use the library, we lock it
+  //!if we currently use the library, we lock it
   int library_locked;
-  //the new filename path (internal)
+  //!the new filename path (internal)
   char *new_filename_path;
 } splt_internal;
 
-/*
+/**
  * Structure containing information about one plugin.
- * Must be filled up at plugin initialisation.
+ * Is filled with values at plugin initialisation.
  */
 typedef struct
 {
@@ -638,7 +720,7 @@ typedef struct
   char *upper_extension;
 } splt_plugin_info;
 
-//contains pointers to the plugin functions
+//!contains pointers to the plugin functions
 typedef struct {
   int (*check_plugin_is_for_file)(void *state, int *error);
   void (*set_plugin_info)(splt_plugin_info *info, int *error);
@@ -649,104 +731,113 @@ typedef struct {
   double (*split)(void *state, const char *final_fname, double begin_point,
       double end_point, int *error, int save_end_point);
   int (*scan_silence)(void *state, int *error);
+  int (*scan_trim_silence)(void *state, int *error);
   void (*set_original_tags)(void *state, int *error);
+  void (*clear_original_tags)(splt_original_tags *original_tags);
   void (*init)(void *state, int *error);
   void (*end)(void *state, int *error);
 } splt_plugin_func;
 
-//structure containing all the data about one plugin
+//!structure containing all the data about a plugin
 typedef struct
 {
   splt_plugin_info info;
-  //complete filename of the plugin shared object
+  //!complete filename of the plugin shared object
   char *plugin_filename;
-  //plugin handle get with lt_dlopen
-  //-would be closed with lt_dlclose
+  //! plugin handle get with lt_dlopen, used later for lt_dlclose
   void *plugin_handle;
-  //plugin functions
+  //! plugin functions
   splt_plugin_func *func;
 } splt_plugin_data;
 
-//internal plugins structure
+//!internal plugins structure
 typedef struct
 {
-  //directories where we scan for plugins
+  //!directories where we scan for plugins
   char **plugins_scan_dirs;
   int number_of_dirs_to_scan;
-  //the number of plugins found
+  //!the number of plugins found
   int number_of_plugins_found;
-  //data structure about all the plugins
+  //!data structure about all the plugins
   splt_plugin_data *data;
 } splt_plugins;
 
-//structure containing error strings for error messages
+//!structure containing error strings for error messages
 typedef struct
 {
   char *error_data;
   char *strerror_msg;
 } splt_error;
 
-//structure for the splt state
+//!structure for the splt state
 typedef struct {
 
-  //if we cancel split or not
-  //set to SPLT_TRUE cancels the split
+  //!setting to SPLT_TRUE cancels the split
   int cancel_split;
-  //filename to split
+  //!filename to split
   char *fname_to_split;
-  //where the split file will be split
+  //!where the split file will be split
   char *path_of_split;
 
   //if this is non null, we write a m3u from the split files
   char *m3u_filename;
+  
+  //!setting tags from input filename regex
+  char *input_fname_regex;
 
-  //tags of the original file to split
-  splt_tags original_tags;
+  char *default_comment_tag;
+  char *default_genre_tag;
 
-  //options for the split
+  //!tags of the original file to split
+  splt_original_tags original_tags;
+
+  //!options for the split
   splt_options options;
-  //split related
+  //!split related
   splt_struct split;
-  //output format  
+  //!output format  
   splt_oformat oformat;
-  //wrap related
+  //!wrap related
   splt_wrap *wrap;
-  //syncerror related
+  //!syncerror related
   splt_syncerrors *serrors;
-  //counter for the number of sync errors found
-  //-the state->serros->serrors_points_num must be used when processing the
-  //syncerrors from 'serrors' 
-  //-this is just a standalone counter
+  /*! counter for the number of sync errors found
+
+    - the state->serros->serrors_points_num must be used when processing the
+      syncerrors from 'serrors' 
+    - this is just a standalone counter
+  */
   unsigned long syncerrors;
-  //freedb related
+  //!freedb related
   splt_freedb fdb;
 
-  //internal options
+  //!internal options
   splt_internal iopts;
 
-  //see the ssplit structure
+  //!see the ssplit structure
   struct splt_ssplit *silence_list;
 
   //proxy infos
   //splt_proxy proxy;
 
-  //file format states, mp3,ogg..
+  //!file format states, mp3,ogg..
   void *codec;
 
-  //error strings for error code messages
+  //!error strings for error code messages
   splt_error err;
 
-  //plugins structure
+  //!plugins structure
   splt_plugins *plug;
   int current_plugin;
 
-  //filename of the silence log: 'mp3splt.log' in the original mp3splt
+  //!filename of the silence log: 'mp3splt.log' in the original mp3splt
   char *silence_log_fname;
 } splt_state;
 
 /*****************************************/
 /* Confirmations, errors and messages    */
 
+//! All error codes we can generate
 typedef enum {
   SPLT_OK = 0,
 
@@ -757,6 +848,7 @@ typedef enum {
   SPLT_NO_SILENCE_SPLITPOINTS_FOUND = 7,
   SPLT_OK_SPLIT_EOF = 8,
   SPLT_LENGTH_SPLIT_OK = 9,
+  SPLT_TRIM_SILENCE_OK = 10,
 
   SPLT_FREEDB_OK = 100,
   SPLT_FREEDB_FILE_OK = 101,
@@ -777,6 +869,8 @@ typedef enum {
   SPLT_OUTPUT_FORMAT_OK = 400,
   SPLT_OUTPUT_FORMAT_AMBIGUOUS = 401,
 
+  SPLT_REGEX_OK = 800,
+
   SPLT_ERROR_SPLITPOINTS = -1,
   SPLT_ERROR_CANNOT_OPEN_FILE = -2,
   SPLT_ERROR_INVALID = -3,
@@ -807,6 +901,7 @@ typedef enum {
   SPLT_ERROR_TIME_SPLIT_VALUE_INVALID = -34,
   SPLT_ERROR_LENGTH_SPLIT_VALUE_INVALID = -35,
   SPLT_ERROR_CANNOT_GET_TOTAL_TIME = -36,
+  SPLT_ERROR_LIBID3 = -37,
 
   SPLT_FREEDB_ERROR_INITIALISE_SOCKET = -101,
   SPLT_FREEDB_ERROR_CANNOT_GET_HOST = -102,
@@ -824,6 +919,7 @@ typedef enum {
   SPLT_INVALID_CDDB_FILE = -116,
   SPLT_FREEDB_NO_SUCH_CD_IN_DATABASE = -118,
   SPLT_FREEDB_ERROR_SITE = -119,
+  SPLT_FREEDB_ERROR_CANNOT_DISCONNECT = -120,
 
   SPLT_DEWRAP_ERR_FILE_LENGTH = -200,
   SPLT_DEWRAP_ERR_VERSION_OLD = -201,
@@ -838,6 +934,9 @@ typedef enum {
   SPLT_PLUGIN_ERROR_UNSUPPORTED_FEATURE = -600,
 
   SPLT_INVALID_AUDACITY_FILE = -700,
+
+  SPLT_INVALID_REGEX = -800,
+  SPLT_REGEX_NO_MATCH = -801,
 } splt_code;
 
 //internal
@@ -1032,8 +1131,8 @@ typedef enum {
   /**
    * The threshold  level (dB) to be considered silence\n
    * It is a float number between -96 and 0. It is used when
-   * having a #SPLT_OPTION_SILENCE_MODE split or when having the
-   * #SPLT_OPT_AUTO_ADJUST option
+   * having a #SPLT_OPTION_SILENCE_MODE or a #SPLT_OPTION_TRIM_SILENCE_MODE split
+   * or when having the #SPLT_OPT_AUTO_ADJUST option
    *
    * The option can take float values between -96 and 0
    *
@@ -1042,8 +1141,8 @@ typedef enum {
   SPLT_OPT_PARAM_THRESHOLD,
   /**
    * Allows you to adjust the offset of cutpoint in silence time when
-   * having a #SPLT_OPTION_SILENCE_MODE split or when having the
-   * #SPLT_OPT_AUTO_ADJUST option
+   * having a #SPLT_OPTION_SILENCE_MODE or a #SPLT_OPTION_TRIM_SILENCE_MODE split
+   * or when having the #SPLT_OPT_AUTO_ADJUST option
    *
    * The option can take float values between -2 and 2\n
    * 0  is  the begin of silence, and 1 the end
@@ -1059,9 +1158,47 @@ typedef enum {
    *
    * Default is #SPLT_DEFAULT_PARAM_MINIMUM_LENGTH
    */
-  SPLT_OPT_PARAM_MIN_LENGTH
+  SPLT_OPT_PARAM_MIN_LENGTH,
+  /**
+   * Format of the artist tag from filename.
+   * Possible values are #splt_str_format
+   */
+  SPLT_OPT_ARTIST_TAG_FORMAT,
+  /**
+   * Format of the album tag from filename.
+   * Possible values are #splt_str_format
+   */
+  SPLT_OPT_ALBUM_TAG_FORMAT,
+  /**
+   * Format of the title tag from filename.
+   * Possible values are #splt_str_format
+   */
+  SPLT_OPT_TITLE_TAG_FORMAT,
+  /**
+   * Format of the comment tag from filename.
+   * Possible values are #splt_str_format
+   */
+  SPLT_OPT_COMMENT_TAG_FORMAT,
+  /**
+   * Replace underscores with space when setting tags
+   * from filename regex.
+   */
+  SPLT_OPT_REPLACE_UNDERSCORES_TAG_FORMAT,
+  /**
+   * If SPLT_TRUE, parse the FILE line and set the filename to split
+   * from the CUE file.
+   */
+  SPLT_OPT_SET_FILE_FROM_CUE_IF_FILE_TAG_FOUND,
 } splt_int_options;
 
+typedef enum {
+  SPLT_NO_CONVERSION,
+  SPLT_TO_LOWERCASE,
+  SPLT_TO_UPPERCASE,
+  SPLT_TO_FIRST_UPPERCASE,
+  SPLT_TO_WORD_FIRST_UPPERCASE
+} splt_str_format;
+
 /**
  * we define a 'skippoint' as a splitpoint that is not taken into
  * consideration
@@ -1073,6 +1210,51 @@ typedef enum {
   SPLT_SKIPPOINT,
 } splt_type_of_splitpoint;
 
+
+#define SPLT_UNDEFINED_GENRE "Other"
+
+#define SPLT_ID3V1_NUMBER_OF_GENRES 127
+
+/*! ID3v1 genres
+*/
+static const char splt_id3v1_genres[SPLT_ID3V1_NUMBER_OF_GENRES][25] = {
+  {"Blues"},
+  {"Classic Rock"}, {"Country"}, {"Dance"}, 
+  {"Disco"},{"Funk"},{"Grunge"},{"Hip-Hop"},{"Jazz"},
+  {"Metal"},{"New Age"},{"Oldies"}, {"Other"}, {"Pop"},
+  {"R&B"}, {"Rap"}, {"Reggae"}, {"Rock"}, {"Techno"},
+  {"Industrial"}, {"Alternative"}, {"Ska"}, {"Death metal"},
+  {"Pranks"}, {"Soundtrack"}, {"Euro-Techno"},
+  {"Ambient"}, {"Trip-hop"}, {"Vocal"}, {"Jazz+Funk"},
+  {"Fusion"}, {"Trance"}, {"Classical"}, {"Instrumental"},
+  {"Acid"}, {"House"}, {"Game"}, {"Sound clip"}, {"Gospel"},
+  {"Noise"}, {"Alt. Rock"}, {"Bass"}, {"Soul"}, {"Punk"}, 
+  {"Space"}, {"Meditative"}, {"Instrumental pop"}, 
+  {"Instrumental rock"}, {"Ethnic"}, {"Gothic"},{"Darkwave"},
+  {"Techno-Industrial"},{"Electronic"},{"Pop-Folk"},{"Eurodance"},
+  {"Dream"},{"Southern Rock"},{"Comedy"}, {"Cult"},{"Gangsta"},
+  {"Top 40"},{"Christian Rap"},{"Pop/Funk"}, {"Jungle"},
+  {"Native American"},{"Cabaret"},{"New Wave"}, {"Psychedelic"},
+  {"Rave"},{"Showtunes"},{"Trailer"}, {"Lo-Fi"},{"Tribal"},
+  {"Acid Punk"},{"Acid Jazz"}, {"Polka"}, {"Retro"},
+  {"Musical"},{"Rock & Roll"},{"Hard Rock"},
+
+  {"Folk"}, {"Folk-Rock"}, {"National Folk"}, {"Swing"},
+  {"Fast Fusion"}, {"Bebob"}, {"Latin"}, {"Revival"},
+  {"Celtic"}, {"Bluegrass"}, {"Avantgarde"}, {"Gothic Rock"},
+  {"Progressive Rock"}, {"Psychedelic Rock"}, {"Symphonic Rock"},
+  {"Slow Rock"}, {"Big Band"}, {"Chorus"}, {"Easy Listening"},
+  {"Acoustic"}, {"Humour"}, {"Speech"}, {"Chanson"}, {"Opera"},
+  {"Chamber Music"}, {"Sonata"}, {"Symphony"}, {"Booty Bass"},
+  {"Primus"}, {"Porn Groove"}, {"Satire"}, {"Slow Jam"},
+  {"Club"}, {"Tango"}, {"Samba"}, {"Folklore"}, {"Ballad"},
+  {"Power Ballad"}, {"Rhythmic Soul"}, {"Freestyle"}, {"Duet"},
+  {"Punk Rock"}, {"Drum Solo"}, {"A capella"}, {"Euro-House"},
+  {"Dance Hall"},
+
+  {"misc"},
+};
+
 /**
  * Freedb constants
  */
@@ -1121,7 +1303,7 @@ typedef enum {
 /**
  * @brief Package version
  */
-#define SPLT_PACKAGE_VERSION "0.6"
+#define SPLT_PACKAGE_VERSION "0.7"
 #endif
 /**
  * @brief Package authors
@@ -1178,49 +1360,130 @@ int mp3splt_find_plugins(splt_state *state);
 void mp3splt_free_state(splt_state *state, int *error);
 
 /************************************/
-/* Set path                         */
+/** \defgrout splt_filepaths Set filenames and paths
 
-//puts the path for the new split files
-//returns possible error
+@{
+*/
+
+/** Sets the path for the creation of new split files
+
+@param state The central struct libmp3splt keeps all its data in
+@param path The new output path
+@return The error code
+ */
 int mp3splt_set_path_of_split(splt_state *state, const char *path);
 
-/************************************/
-/* Set filename                     */
+/** Sets path to the input file
 
-//put the filename to split
-//returns possible error
+@param state The central struct libmp3splt keeps all its data in
+@param path The new input file name
+@return The error code
+ */
 int mp3splt_set_filename_to_split(splt_state *state, const char *filename);
+
+/** Reads out the path to the input file
+
+Is very practical after importing a cue file: The cue file provides an
+input filename but does not tell us about this.
+
+@param state The central struct libmp3splt keeps all its data in
+@param path The new input file name
+@return The error code
+ */
+char *mp3splt_get_filename_to_split(splt_state *state);
+
 int mp3splt_set_m3u_filename(splt_state *state, const char *filename);
 int mp3splt_set_silence_log_filename(splt_state *state, const char *filename);
 
-/************************************/
-/* Set callback functions           */
+/**
+ * Defines the regex that will be used for #SPLT_TAGS_FROM_FILENAME_REGEX
+ * to get out tags.
+ *
+ * Example: (?<artist>.*?) _ (?<album>.*?) will extract
+ * 'one artist' and 'one album' from 'one artist _ one album',
+ */
+int mp3splt_set_input_filename_regex(splt_state *state, const char *regex);
 
+int mp3splt_set_default_comment_tag(splt_state *state, const char *default_comment_tag);
+
+int mp3splt_set_default_genre_tag(splt_state *state, const char *default_genre_tag);
+
+//@}
+
+/************************************/
+/*! @defgroup splt_callback_ The registering of callback functions
+
+@{
+ */
+
+/** Register the function that is to be called if we got a text message
+
+@param state The central struct libmp3splt keeps all its data in
+@param message_cb The callback function
+@return The error code
+ */
 int mp3splt_set_message_function(splt_state *state, 
     void (*message_cb)(const char *, splt_message_type));
+
+/** Register the function that is to be called on creation of a output file
+
+@param state The central struct libmp3splt keeps all its data in
+@param file_cb The callback function
+@return The error code
+ */
 int mp3splt_set_split_filename_function(splt_state *state,
     void (*file_cb)(const char *,int));
+
+/** Register the progress bar callback
+
+@param state The central struct libmp3splt keeps all its data in
+@param progress_cb The callback function
+@return The error code
+ */
 int mp3splt_set_progress_function(splt_state *state,
     void (*progress_cb)(splt_progress *p_bar));
+
 int mp3splt_set_silence_level_function(splt_state *state,
   void (*get_silence_cb)(long time, float level, void *user_data),
   void *user_data);
 
-/************************************/
-/* Splitpoints                      */
+//!@}
 
-//puts a splitpoint
-//returns possible error
+/************************************/
+/*! @defgroup splt_splitpoints_ Splitpoint manipulation
+
+@{
+ */
+
+/** Sets a new splitpoint
+@param state The central struct libmp3splt keeps all its data in
+@param type @see splt_type_of_splitpoint
+@param split_value The position of the split point in 1/100s
+@param name The name of the split point
+@return The error code of any error that might have occoured
+*/
 int mp3splt_append_splitpoint(splt_state *state,
     long split_value, const char *name, int type);
 
-//returns a pointer to all the current splitpoints
+/** returns a pointer to a list of all splitpoints
+
+@param state The central struct libmp3splt keeps all its data in
+@param error Is filled with the error code of any error that might
+             have occoured
+@param splitpoints_number The number of split points in the list
+@return A pointer to the split point list
+ */
 const splt_point *mp3splt_get_splitpoints(splt_state *state,
     int *splitpoints_number, int *error);
 
-//erase all the splitpoints
+/** erase all splitpoints
+@param state The central struct libmp3splt keeps all its data in
+@param error Is filled with the error code of any error that might
+             have occoured
+ */
 void mp3splt_erase_all_splitpoints(splt_state *state,
     int *error);
+//@}
 
 /************************************/
 /* Tags                             */
@@ -1230,7 +1493,7 @@ int mp3splt_append_tags(splt_state *state,
     const char *title, const char *artist,
     const char *album, const char *performer,
     const char *year, const char *comment,
-    int track, unsigned char genre);
+    int track, const char *genre);
 
 //returns a pointer to all the current tags
 const splt_tags *mp3splt_get_tags(splt_state *state,
@@ -1341,6 +1604,9 @@ char **mp3splt_find_filenames(splt_state *state, const char *filename,
 
 int mp3splt_u_check_if_directory(const char *fname);
 
+void mp3splt_free_one_tag(splt_tags *tags);
+splt_tags *mp3splt_parse_filename_regex(splt_state *state, int *error);
+
 #define MP3SPLT_MP3SPLT_H
 
 #endif
