@@ -5,7 +5,7 @@
  *
  * Copyright (c) 2002-2004 M. Trotta - <matteo.trotta@lib.unimib.it>
  * Copyright (c) 2007 Federico Grau - <donfede@casagrau.org>
- * Copyright (c) 2010 Alexandru Munteanu <io_fx@yahoo.fr>
+ * Copyright (c) 2010-2011 Alexandru Munteanu <io_fx@yahoo.fr>
  *
  * http://mp3splt.sourceforge.net
  * http://audacity.sourceforge.net/
@@ -31,6 +31,11 @@
  *
  *********************************************************/
 
+/*!\file 
+
+Input splitpoints from audacity.
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -40,22 +45,156 @@
 
 #include "splt.h"
 
+
+static long splt_audacity_get_begin(splt_audacity *sa)
+{
+  return sa->begin;
+}
+
+static long splt_audacity_get_end(splt_audacity *sa)
+{
+  return sa->end;
+}
+
+static const char *splt_audacity_get_name(splt_audacity *sa)
+{
+  return sa->name;
+}
+
 static int splt_audacity_append_splitpoints(splt_state *state,
-    splt_audacity *previous_aud, splt_audacity *aud, int *append_start_point);
+    splt_audacity *previous_aud, splt_audacity *aud, int *append_begin_point)
+{
+  int err = SPLT_OK;
+
+  long previous_begin_point = -1;
+  long previous_end_point = -1;
+
+  if (previous_aud)
+  {
+    previous_begin_point = splt_audacity_get_begin(previous_aud);
+    previous_end_point = splt_audacity_get_end(previous_aud);
+  }
+
+  long start_point = splt_audacity_get_begin(aud);
+
+  if (previous_begin_point != -1 && previous_end_point != -1)
+  {
+    if (*append_begin_point)
+    {
+      err = splt_sp_append_splitpoint(state, previous_begin_point, 
+          splt_audacity_get_name(previous_aud), SPLT_SPLITPOINT);
+      if (err < 0) { return err; }
+    }
+
+    if (start_point == previous_end_point)
+    {
+      err = splt_sp_append_splitpoint(state, previous_end_point, 
+          splt_audacity_get_name(aud), SPLT_SPLITPOINT);
+      *append_begin_point = SPLT_FALSE;
+      if (err < 0) { return err; }
+    }
+    else
+    {
+      err = splt_sp_append_splitpoint(state, previous_end_point, "skip", SPLT_SKIPPOINT);
+      *append_begin_point = SPLT_TRUE;
+      if (err < 0) { return err; }
+    }
+  }
+
+  return err;
+}
+
+static splt_audacity *splt_audacity_new()
+{
+  splt_audacity *sa = malloc(sizeof(splt_audacity));
+  if (!sa)
+  {
+    return NULL;
+  }
+
+  sa->begin = -1;
+  sa->end = -1;
+  sa->name = NULL;
+
+  return sa;
+}
+
+static void splt_audacity_free(splt_audacity **sa)
+{
+  if (sa)
+  {
+    if (*sa)
+    {
+      splt_audacity *s = *sa;
+
+      if (s->name)
+      {
+        free(s->name);
+        s->name = NULL;
+      }
+
+      free(*sa);
+      *sa = NULL;
+    }
+  }
+}
+
+static int splt_audacity_set_name(splt_audacity *sa, const char *name)
+{
+  return splt_su_copy(name, &sa->name);
+}
+
+static void splt_audacity_set_begin(splt_audacity *sa, double begin)
+{
+  sa->begin = (long) (floorf(begin) * 100.0);
+}
+
+static void splt_audacity_set_end(splt_audacity *sa, double end)
+{
+  sa->end = (long) (floorf(end) * 100.0);
+}
 
 static splt_audacity *splt_audacity_process_line(splt_state *state, char *line,
-    splt_audacity *previous_aud, int *append_start_point, int *error);
+    splt_audacity *previous_aud, int *append_begin_point, int *error)
+{
+  splt_audacity *aud = splt_audacity_new();
+  if (!aud)
+  {
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    return NULL;
+  }
 
-static splt_audacity *splt_audacity_new();
-static void splt_audacity_free(splt_audacity **sa);
+  char *ptr = line;
 
-static int splt_audacity_set_name(splt_audacity *sa, const char *name);
-static void splt_audacity_set_begin(splt_audacity *sa, double begin);
-static void splt_audacity_set_end(splt_audacity *sa, double end);
+  errno = 0;
+  splt_audacity_set_begin(aud, strtod(ptr, &ptr));
+  if (errno != 0) {
+    *error = SPLT_INVALID_AUDACITY_FILE;
+    goto error;
+  }
 
-static long splt_audacity_get_begin(splt_audacity *sa);
-static long splt_audacity_get_end(splt_audacity *sa);
-static const char *splt_audacity_get_name(splt_audacity *sa);
+  errno = 0;
+  splt_audacity_set_end(aud, strtod(ptr, &ptr));
+  if (errno != 0)
+  {
+    *error = SPLT_INVALID_AUDACITY_FILE;
+    goto error;
+  }
+
+  ptr = splt_su_trim_spaces(ptr);
+
+  int err = splt_audacity_set_name(aud, ptr);
+  if (err < 0) { *error = err; goto error; }
+
+  err = splt_audacity_append_splitpoints(state, previous_aud, aud, append_begin_point);
+  if (err < 0) { *error = err; goto error; }
+
+  return aud;
+
+error:
+  splt_audacity_free(&aud);
+  return NULL;
+}
 
 int splt_audacity_put_splitpoints(const char *file, splt_state *state, int *error)
 {
@@ -153,153 +292,3 @@ end:
 	return tracks;
 }
 
-static splt_audacity *splt_audacity_process_line(splt_state *state, char *line,
-    splt_audacity *previous_aud, int *append_begin_point, int *error)
-{
-  splt_audacity *aud = splt_audacity_new();
-  if (!aud)
-  {
-    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-    return NULL;
-  }
-
-  char *ptr = line;
-
-  errno = 0;
-  splt_audacity_set_begin(aud, strtod(ptr, &ptr));
-  if (errno != 0) {
-    *error = SPLT_INVALID_AUDACITY_FILE;
-    goto error;
-  }
-
-  errno = 0;
-  splt_audacity_set_end(aud, strtod(ptr, &ptr));
-  if (errno != 0)
-  {
-    *error = SPLT_INVALID_AUDACITY_FILE;
-    goto error;
-  }
-
-  ptr = splt_su_trim_spaces(ptr);
-
-  int err = splt_audacity_set_name(aud, ptr);
-  if (err < 0) { *error = err; goto error; }
-
-  err = splt_audacity_append_splitpoints(state, previous_aud, aud, append_begin_point);
-  if (err < 0) { *error = err; goto error; }
-
-  return aud;
-
-error:
-  splt_audacity_free(&aud);
-  return NULL;
-}
-
-static int splt_audacity_append_splitpoints(splt_state *state,
-    splt_audacity *previous_aud, splt_audacity *aud, int *append_begin_point)
-{
-  int err = SPLT_OK;
-
-  long previous_begin_point = -1;
-  long previous_end_point = -1;
-
-  if (previous_aud)
-  {
-    previous_begin_point = splt_audacity_get_begin(previous_aud);
-    previous_end_point = splt_audacity_get_end(previous_aud);
-  }
-
-  long start_point = splt_audacity_get_begin(aud);
-
-  if (previous_begin_point != -1 && previous_end_point != -1)
-  {
-    if (*append_begin_point)
-    {
-      err = splt_sp_append_splitpoint(state, previous_begin_point, 
-          splt_audacity_get_name(previous_aud), SPLT_SPLITPOINT);
-      if (err < 0) { return err; }
-    }
-
-    if (start_point == previous_end_point)
-    {
-      err = splt_sp_append_splitpoint(state, previous_end_point, 
-          splt_audacity_get_name(aud), SPLT_SPLITPOINT);
-      *append_begin_point = SPLT_FALSE;
-      if (err < 0) { return err; }
-    }
-    else
-    {
-      err = splt_sp_append_splitpoint(state, previous_end_point, "skip", SPLT_SKIPPOINT);
-      *append_begin_point = SPLT_TRUE;
-      if (err < 0) { return err; }
-    }
-  }
-
-  return err;
-}
-
-static splt_audacity *splt_audacity_new()
-{
-  splt_audacity *sa = malloc(sizeof(splt_audacity));
-  if (!sa)
-  {
-    return NULL;
-  }
-
-  sa->begin = -1;
-  sa->end = -1;
-  sa->name = NULL;
-
-  return sa;
-}
-
-static void splt_audacity_free(splt_audacity **sa)
-{
-  if (sa)
-  {
-    if (*sa)
-    {
-      splt_audacity *s = *sa;
-
-      if (s->name)
-      {
-        free(s->name);
-        s->name = NULL;
-      }
-
-      free(*sa);
-      *sa = NULL;
-    }
-  }
-}
-
-static int splt_audacity_set_name(splt_audacity *sa, const char *name)
-{
-  return splt_su_copy(name, &sa->name);
-}
-
-static void splt_audacity_set_begin(splt_audacity *sa, double begin)
-{
-  sa->begin = (long) (floorf(begin) * 100.0);
-}
-
-static void splt_audacity_set_end(splt_audacity *sa, double end)
-{
-  sa->end = (long) (floorf(end) * 100.0);
-}
-
-static long splt_audacity_get_begin(splt_audacity *sa)
-{
-  return sa->begin;
-}
-
-static long splt_audacity_get_end(splt_audacity *sa)
-{
-  return sa->end;
-}
-
-static const char *splt_audacity_get_name(splt_audacity *sa)
-{
-  return sa->name;
-}
-
