@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -30,6 +30,12 @@
  *
  *********************************************************/
 
+/*! Error handling
+
+Don't use these functions directly. The version of these functions
+that is meant to be used directly are all in mp3splt.c.
+*/
+
 #include <string.h>
 #include <errno.h>
 
@@ -203,11 +209,15 @@ char *splt_e_strerror(splt_state *state, splt_code error_code)
           state->err.error_data, state->err.strerror_msg);
     case SPLT_FREEDB_ERROR_CANNOT_OPEN_SOCKET:
       return splt_su_get_formatted_message(state,
-          _(" freedb error: cannot open socket"));
+          _(" freedb error: cannot open socket (%s)"));
     case SPLT_FREEDB_ERROR_CANNOT_CONNECT:
       return splt_su_get_formatted_message(state,
           _(" freedb error: cannot connect to host '%s' (%s)"),
           state->err.error_data, state->err.strerror_msg);
+    case SPLT_FREEDB_ERROR_CANNOT_DISCONNECT:
+      return splt_su_get_formatted_message(state,
+          _(" freedb error: cannot disconnect from host (%s)"),
+          state->err.strerror_msg);
     case SPLT_FREEDB_ERROR_CANNOT_SEND_MESSAGE:
       return splt_su_get_formatted_message(state,
           _(" freedb error: cannot send message to host '%s' (%s)"),
@@ -276,6 +286,8 @@ char *splt_e_strerror(splt_state *state, splt_code error_code)
       return splt_su_get_formatted_message(state, _(" split in equal tracks ok"));
     case SPLT_SILENCE_OK:
       return splt_su_get_formatted_message(state, _(" silence split ok"));
+    case SPLT_TRIM_SILENCE_OK:
+      return splt_su_get_formatted_message(state, _(" trim using silence split ok"));
     case SPLT_SPLITPOINT_BIGGER_THAN_LENGTH:
       return splt_su_get_formatted_message(state,
           _(" file split, splitpoints bigger than length"));
@@ -312,6 +324,9 @@ char *splt_e_strerror(splt_state *state, splt_code error_code)
     case SPLT_ERROR_CANNOT_GET_TOTAL_TIME:
       return splt_su_get_formatted_message(state,
           _(" error: cannot get total audio length"));
+    case SPLT_ERROR_LIBID3:
+      return splt_su_get_formatted_message(state,
+          _(" error: unknown error while setting tags with libid3"));
     case SPLT_ERROR_SPLITPOINTS_NOT_IN_ORDER:
       return splt_su_get_formatted_message(state,
           _(" error: the splitpoints are not in order (%s)"),
@@ -413,6 +428,13 @@ char *splt_e_strerror(splt_state *state, splt_code error_code)
       return splt_su_get_formatted_message(state,
           _(" audacity error: invalid audacity labels file '%s'"),
           state->err.error_data);
+    case SPLT_INVALID_REGEX:
+      return splt_su_get_formatted_message(state,
+          _(" regular expression error: %s"), state->err.error_data);
+    case SPLT_REGEX_NO_MATCH:
+      return splt_su_get_formatted_message(state, _(" regular expression error: no match"));
+    case SPLT_REGEX_OK:
+      return splt_su_get_formatted_message(state, _(" regular expression ok"));
   }
 
   return NULL;
