@@ -30,20 +30,71 @@
  *
  *********************************************************/
 
+/*! \file 
+
+All user-accessible functions
+
+The functions that are actually meant to be called by the user. If
+this was C++ it would be the "public" section of our object.
+
+Actually this whole library seems to be written with object orientied
+programming in mind: All functions expect the address of the object as
+the first parameter etc. Actually changing the library to be c++
+should therefore not be too hard a task...
+ */
+
+/*! \mainpage Using libmp3splt
+
+This is the documentation for the library that stands behind
+mp3splt-gtk and mp3splt.\n
+Features include
+ - losslessly editing mp3 and og files
+ - extensibility to other formats using plug-ins
+ - freedb search
+
+Documentation can be found at several points:
+- A minimal example on how to use this library can be found in \ref test_library.c
+- There is a \ref API
+- For plug-in writers there is a \ref PluginAPI_MP3
+- And --- also for plugin-writers there is a \ref PluginAPI_MP3 as well.
+ */
+
+/*!   \example test_library.c
+
+  A minimal usage example
+ */
+
 #include <sys/stat.h>
 #include <string.h>
 
+#include <ltdl.h>
+
 #include "splt.h"
 
 int global_debug = SPLT_FALSE;
 
-/************************************/
-/* Initialisation and free          */
+/*! 
+\defgroup API The API of libmp3splt
 
-/**
- * creates and returns the new mp3splt state
- * \param error A possible error.
- */
+This group contains all functions of libmp3splt that are meant to be
+accessed by the user
+
+@{
+*/
+
+/*! Initialisation
+
+ creates and returns the new central splt_state structure this
+ library keeps all its data in. 
+
+ The next step after initialization is to do a
+ mp3splt_find_plugins() to find any plugins;
+
+ At the end of the program all data we allocate here is freed by
+ calling mp3splt_free_state()
+
+ \param error Contains the error code if an error occours.
+*/
 splt_state *mp3splt_new_state(int *error)
 {
   splt_state *state = NULL;
@@ -72,19 +123,31 @@ splt_state *mp3splt_new_state(int *error)
   return state;
 }
 
-//find plugins and initialise them
-//-returns possible error
+/*! find plugins and initialise them
+
+\return The error code if necessary
+*/
 int mp3splt_find_plugins(splt_state *state)
 {
   return splt_p_find_get_plugins_data(state);
 }
 
-//this function frees the left variables in the library
-//call this function ONLY at the end of the program
-//and don't forget to call it
-//Returns possible error in *error
+/*! this function frees all variables malloc'ed by the library
+
+call this function ONLY at the end of the program
+and don't forget to call it.
+
+\param error The code of any error that might have occoured 
+\param state the splt_state structure that keeps all data for 
+ our library
+*/
 void mp3splt_free_state(splt_state *state, int *error)
-{  
+{
+  if (!state)
+  {
+    return;
+  }
+
   int erro = SPLT_OK;
   int *err = &erro;
   if (error != NULL) { err = error; }
@@ -108,9 +171,7 @@ void mp3splt_free_state(splt_state *state, int *error)
   }
 }
 
-/************************************/
-/* Set path                         */
-
+//! Set the output path
 int mp3splt_set_path_of_split(splt_state *state, const char *path)
 {
   int error = SPLT_OK;
@@ -138,10 +199,11 @@ int mp3splt_set_path_of_split(splt_state *state, const char *path)
   return error;
 }
 
-/************************************/
-/* Set filename                     */
+/*! Set the m3u filename
 
-//sets the m3u filename
+\param state the splt_state structure that keeps all our data
+\param filename Constains the file name that is to be set
+ */
 int mp3splt_set_m3u_filename(splt_state *state, const char *filename)
 {
   int error = SPLT_OK;
@@ -169,7 +231,7 @@ int mp3splt_set_m3u_filename(splt_state *state, const char *filename)
   return error;
 }
 
-//sets the m3u filename
+//!sets the m3u filename
 int mp3splt_set_silence_log_filename(splt_state *state, const char *filename)
 {
   int error = SPLT_OK;
@@ -197,8 +259,20 @@ int mp3splt_set_silence_log_filename(splt_state *state, const char *filename)
   return error;
 }
 
+/*! \brief Reads out the name of the file that is about to be split
 
-//puts the filename to split in the state
+  
+ */
+char *mp3splt_get_filename_to_split(splt_state *state)
+{
+  return splt_t_get_filename_to_split(state); 
+}
+
+/*! \brief Sets the name of the file that is about to be split
+
+\param state The central data structure this library keeps its data in
+\param filename The filename we want to save in state
+*/
 int mp3splt_set_filename_to_split(splt_state *state, const char *filename)
 {
   int error = SPLT_OK;
@@ -226,11 +300,97 @@ int mp3splt_set_filename_to_split(splt_state *state, const char *filename)
   return error;
 }
 
+int mp3splt_set_input_filename_regex(splt_state *state, const char *regex)
+{
+  int error = SPLT_OK;
+
+  if (state != NULL)
+  {
+    if (!splt_o_library_locked(state))
+    {
+      splt_o_lock_library(state);
+
+      error = splt_t_set_input_filename_regex(state, regex);
+
+      splt_o_unlock_library(state);
+    }
+    else
+    {
+      error = SPLT_ERROR_LIBRARY_LOCKED;
+    }
+  }
+  else
+  {
+    error = SPLT_ERROR_STATE_NULL;
+  }
+
+  return error;
+}
+
+int mp3splt_set_default_comment_tag(splt_state *state, const char *default_comment)
+{
+  int error = SPLT_OK;
+
+  if (state != NULL)
+  {
+    if (!splt_o_library_locked(state))
+    {
+      splt_o_lock_library(state);
+
+      error = splt_t_set_default_comment_tag(state, default_comment);
+
+      splt_o_unlock_library(state);
+    }
+    else
+    {
+      error = SPLT_ERROR_LIBRARY_LOCKED;
+    }
+  }
+  else
+  {
+    error = SPLT_ERROR_STATE_NULL;
+  }
+
+  return error;
+}
+
+int mp3splt_set_default_genre_tag(splt_state *state, const char *default_genre_tag)
+{
+  int error = SPLT_OK;
+
+  if (state != NULL)
+  {
+    if (!splt_o_library_locked(state))
+    {
+      splt_o_lock_library(state);
+
+      error = splt_t_set_default_genre_tag(state, default_genre_tag);
+
+      splt_o_unlock_library(state);
+    }
+    else
+    {
+      error = SPLT_ERROR_LIBRARY_LOCKED;
+    }
+  }
+  else
+  {
+    error = SPLT_ERROR_STATE_NULL;
+  }
+
+  return error;
+}
+
+
 /************************************/
 /* Set callback functions           */
 
-//sets the function that sends messages to the client
-//returns possible error
+/*! Register the callback that is called for messages to the client
+
+\param state The central structure this library keeps all its data in
+\param message_cb The callback function
+\return The error code if any error occours
+*/
 int mp3splt_set_message_function(splt_state *state, 
     void (*message_cb)(const char *, splt_message_type))
 {
@@ -248,8 +408,12 @@ int mp3splt_set_message_function(splt_state *state,
   return error;
 }
 
-//sets the function that sends the split filename after a split
-//returns possible error
+/*! Register the callback called with the split filename after split
+
+\param state The central structure this library keeps all its data in
+\param file_cb The callback function
+\return The error code if any error occours
+*/
 int mp3splt_set_split_filename_function(splt_state *state,
     void (*file_cb)(const char *,int b))
 {
@@ -267,7 +431,12 @@ int mp3splt_set_split_filename_function(splt_state *state,
   return error;
 }
 
-//sets the function that sends progress messages to the client
+/*! Register the callback called with progress messages
+
+\param state The central structure this library keeps all its data in
+\param progress_cb The callback function
+\return The error code if any error occours
+*/
 int mp3splt_set_progress_function(splt_state *state,
     void (*progress_cb)(splt_progress *p_bar))
 {
@@ -285,7 +454,13 @@ int mp3splt_set_progress_function(splt_state *state,
   return error;
 }
 
-int mp3splt_set_silence_level_function(splt_state *state,
+/*! Register the callback for the function that calculates silence
+    levels 
+
+\param state The central structure this library keeps all its data in
+\param get_silence_cb The callback function
+\return The error code if any error occours
+*/int mp3splt_set_silence_level_function(splt_state *state,
   void (*get_silence_cb)(long time, float level, void *user_data),
   void *data)
 {
@@ -307,9 +482,15 @@ int mp3splt_set_silence_level_function(splt_state *state,
 /************************************/
 /* Splitpoints                      */
 
-//puts a splitpoint in the state with an eventual file name
-//split_value is which splitpoint hundreths of seconds
-//if split_value is LONG_MAX, we put the end of the song (EOF)
+/*! puts a splitpoint in the state 
+
+\param state The central data structure this library keeps all its
+data in
+\param name The file name if we want to set it, else NULL
+\param split_value The time of this splitpoint in hundreths of
+seconds. If this walue is LONG_MAX we put the splitpoint to the end of
+the song (EOF)
+*/
 int mp3splt_append_splitpoint(splt_state *state,
     long split_value, const char *name, int type)
 {
@@ -338,7 +519,13 @@ int mp3splt_append_splitpoint(splt_state *state,
   return error;
 }
 
-//returns a list containing all the splitpoints
+/*!returns a list containing all the splitpoints
+
+\param state The structure this library keeps all its data in
+\param splitpoints_number Will be set to the number of splitpoints by
+this function
+\param error Is set to the error code if any error occours
+ */
 const splt_point *mp3splt_get_splitpoints(splt_state *state,
     int *splitpoints_number, int *error)
 {
@@ -357,8 +544,11 @@ const splt_point *mp3splt_get_splitpoints(splt_state *state,
   }
 }
 
-//erase all the splitpoints
-//returns possible errors
+/*! erase all the splitpoints
+
+\param state The structure this library keeps all its data in
+\param error Is set to the error code if any error occours
+*/
 void mp3splt_erase_all_splitpoints(splt_state *state, int *error)
 {
   int erro = SPLT_OK;
@@ -389,12 +579,12 @@ void mp3splt_erase_all_splitpoints(splt_state *state, int *error)
 /************************************/
 /* Tags                             */
 
-//append tags
+//!append tags
 int mp3splt_append_tags(splt_state *state, 
     const char *title, const char *artist,
     const char *album, const char *performer,
     const char *year, const char *comment,
-    int track, unsigned char genre)
+    int track, const char *genre)
 {
   int error = SPLT_OK;
 
@@ -405,7 +595,7 @@ int mp3splt_append_tags(splt_state *state,
       splt_o_lock_library(state);
 
       error = splt_tu_append_tags(state, title, artist,
-          album, performer, year, comment, track, genre);
+          album, performer, year, comment, track, genre, SPLT_FALSE);
 
       splt_o_unlock_library(state);
     }
@@ -422,7 +612,7 @@ int mp3splt_append_tags(splt_state *state,
   return error;
 }
 
-//returns a list containing all the tags
+//!returns a list containing all the tags
 const splt_tags *mp3splt_get_tags(splt_state *state,
     int *tags_number, int *error)
 {
@@ -441,7 +631,7 @@ const splt_tags *mp3splt_get_tags(splt_state *state,
   }
 }
 
-//puts tags from a string
+//!puts tags from a string
 int mp3splt_put_tags_from_string(splt_state *state, const char *tags, int *error)
 {
   int ambigous = SPLT_FALSE;
@@ -472,7 +662,7 @@ int mp3splt_put_tags_from_string(splt_state *state, const char *tags, int *error
   return ambigous;
 }
 
-//erase all the tags
+//!erase all the tags
 void mp3splt_erase_all_tags(splt_state *state, int *error)
 {
   int erro = SPLT_OK;
@@ -599,9 +789,11 @@ float mp3splt_get_float_option(splt_state *state, int option_name, int *error)
 /************************************/
 /* Split functions                  */
 
-//main function, split the file
-//splitnumber = how many splits
-//returns possible error
+/*! main function: Call it and it splits the file for you
+
+state-> splitnumber tells how many splits to do
+\return the error code if any error has occoured
+*/
 int mp3splt_split(splt_state *state)
 {
   int error = SPLT_OK;
@@ -678,24 +870,6 @@ int mp3splt_split(splt_state *state)
 
       int split_type = splt_o_get_int_option(state, SPLT_OPT_SPLIT_MODE);
 
-      //normal split checks
-      if (split_type == SPLT_OPTION_NORMAL_MODE)
-      {
-        if (! splt_o_get_int_option(state, SPLT_OPT_PRETEND_TO_SPLIT))
-        {
-          //check if we have at least 2 splitpoints
-          if (splt_t_get_splitnumber(state) < 2)
-          {
-            error = SPLT_ERROR_SPLITPOINTS;
-            goto function_end;
-          }
-        }
-
-        //we check if the splitpoints are in order
-        splt_check_if_points_in_order(state, &error);
-        if (error < 0) { goto function_end; }
-      }
-
       splt_t_set_new_filename_path(state, new_filename_path, &error);
       if (error < 0) { goto function_end; }
 
@@ -707,20 +881,23 @@ int mp3splt_split(splt_state *state)
       splt_check_if_fname_path_is_correct(state, new_filename_path, &error);
       if (error < 0) { goto function_end; }
 
-      if (splt_o_get_int_option(state, SPLT_OPT_TAGS) == SPLT_TAGS_ORIGINAL_FILE)
-      {
-        splt_tp_put_tags_from_string(state, SPLT_ORIGINAL_TAGS_DEFAULT, &error);
-        if (error < 0)
-        {
-          splt_p_end(state, &error);
-          goto function_end;
-        }
-      }
-
       //we check if mp3 or ogg
       splt_check_file_type(state, &error);
       if (error < 0) { goto function_end; }
 
+      int tags_option = splt_o_get_int_option(state, SPLT_OPT_TAGS);
+      if (tags_option == SPLT_TAGS_ORIGINAL_FILE)
+      {
+        splt_tp_put_tags_from_string(state, SPLT_ORIGINAL_TAGS_DEFAULT, &error);
+        if (error < 0) { goto function_end; }
+      }
+      else if (tags_option == SPLT_TAGS_FROM_FILENAME_REGEX)
+      {
+        int regex_error = SPLT_OK;
+        splt_tp_put_tags_from_filename(state, &regex_error);
+        if (regex_error < 0) { error = regex_error; goto function_end; }
+      }
+
       const char *plugin_name = splt_p_get_name(state,&error);
       if (error < 0) { goto function_end; }
 
@@ -750,6 +927,7 @@ int mp3splt_split(splt_state *state)
       {
         if ((split_type != SPLT_OPTION_WRAP_MODE)
             && (split_type != SPLT_OPTION_SILENCE_MODE)
+            && (split_type != SPLT_OPTION_TRIM_SILENCE_MODE)
             && (split_type != SPLT_OPTION_ERROR_MODE))
         {
           splt_c_put_info_message_to_client(state, 
@@ -769,6 +947,9 @@ int mp3splt_split(splt_state *state)
         case SPLT_OPTION_SILENCE_MODE:
           splt_s_silence_split(state, &error);
           break; 
+        case SPLT_OPTION_TRIM_SILENCE_MODE:
+          splt_s_trim_silence_split(state, &error);
+          break;
         case SPLT_OPTION_TIME_MODE:
           splt_s_time_split(state, &error);
           break;
@@ -781,7 +962,20 @@ int mp3splt_split(splt_state *state)
         default:
           if (split_type == SPLT_OPTION_NORMAL_MODE)
           {
-            splt_check_points_inf_song_length(state, &error);
+            if (! splt_o_get_int_option(state, SPLT_OPT_PRETEND_TO_SPLIT))
+            {
+              //check if we have at least 2 splitpoints
+              if (splt_t_get_splitnumber(state) < 2)
+              {
+                error = SPLT_ERROR_SPLITPOINTS;
+                goto function_end;
+              }
+            }
+
+            splt_check_points_inf_song_length_and_convert_negatives(state, &error);
+            if (error < 0) { goto function_end; }
+
+            splt_check_if_points_in_order(state, &error);
             if (error < 0) { goto function_end; }
           }
 
@@ -814,8 +1008,9 @@ function_end:
   return error;
 }
 
-//cancels the current split
-//returns possible errors
+/*! cancels the current split
+\return The error code if any error occours
+*/
 void mp3splt_stop_split(splt_state *state, int *error)
 {
   int erro = SPLT_OK;
@@ -835,8 +1030,12 @@ void mp3splt_stop_split(splt_state *state, int *error)
 /************************************/
 /*    Cddb and Cue functions        */
 
-//we get the cue splitpoints from the file
-//returns possible error in err
+/*! Fetch the splitpoints from a cue file
+\param state The central structure that keeps all data this library
+uses 
+\param file The name of the input file
+\param error Is set to the error code this action results in
+*/
 void mp3splt_put_cue_splitpoints_from_file(splt_state *state,
     const char *file, int *error)
 {
@@ -865,7 +1064,13 @@ void mp3splt_put_cue_splitpoints_from_file(splt_state *state,
   }
 }
 
-//we get the cddb splitpoints from the file
+/*! get the cddb splitpoints from the file
+
+\param state The central structure that keeps all data this library
+uses 
+\param file The name of the input file
+\param error Is set to the error code this action results in
+*/
 void mp3splt_put_cddb_splitpoints_from_file(splt_state *state,
     const char *file, int *error)
 {
@@ -894,6 +1099,13 @@ void mp3splt_put_cddb_splitpoints_from_file(splt_state *state,
   }
 }
 
+/*! get the splitpoints from a audacity splitpoint file
+
+\param state The central structure that keeps all data this library
+uses 
+\param file The name of the input file
+\param error Is set to the error code this action results in
+*/
 void mp3splt_put_audacity_labels_splitpoints_from_file(splt_state *state,
     const char *file, int *error)
 {
@@ -925,19 +1137,25 @@ void mp3splt_put_audacity_labels_splitpoints_from_file(splt_state *state,
 /************************************/
 /*    Freedb functions              */
 
-//get freedb search
+/*!Do a freedb search
+
+After dong the search continue by calling
+mp3splt_write_freedb_file_result().
+\param state The central structure this library keeps all its data in
+\param search_string The string that is to be searched for
+\param error The error code this action returns in
+\param search_type the type of the search. Usually set to
+SPLT_SEARCH_TYPE_FREEDB2
+\param search_server The URL of the search server or NULL to select
+the default
+\param port The port on the server. -1 means default (Which should be
+80). 
+ */
 const splt_freedb_results *mp3splt_get_freedb_search(splt_state *state,
-    //our search
     const char *search_string,
-    //possible errors
     int *error,
-    //the type of the search
-    //usually SPLT_SEARCH_TYPE_FREEDB2
     int search_type,
-    //if strlen(search_server) == 0, we put the default
-    //or null
     const char search_server[256],
-    //if port=-1, we use 80
     int port)
 {
   int erro = SPLT_OK;
@@ -977,20 +1195,27 @@ const splt_freedb_results *mp3splt_get_freedb_search(splt_state *state,
   }
 }
 
-//must be called after get_freedb_search,
-//otherwise, it will fail (seg fault!?)
-//result must be freed
-//returns the content of a cddb file
-//you need to write it on the disk in 
-//a cddb file to use it
-//we return possible errors in err
-//the cddb_file is the file to write
-//
-//cddb_get_type specifies the type of the get 
-// -it can be SPLT_FREEDB_GET_FILE_TYPE_CDDB_CGI (that works for both
-//  freedb and freedb2 at the moment - 18_10_06)
-//  or SPLT_FREEDB_GET_FILE_TYPE_CDDB (that only work for freedb at
-//  the moment - 18_10_06)
+/*! returns the content of a cddb file
+
+must only be called \emph{after} running get_freedb_search
+otherwise, it will fail (seg fault!?)
+
+you need to write the cddb entry to the disk in 
+a cddb file to use it be able to use it
+
+\param state The central structure that keeps all data this library
+uses 
+\param error Is set to the error code this action results in
+\param disc_id The freedb disc ID.
+\param cddb_get_type specifies the type of the get:
+  it can be SPLT_FREEDB_GET_FILE_TYPE_CDDB_CGI (that works for both
+  freedb and freedb2 at the moment - 18_10_06)
+  or SPLT_FREEDB_GET_FILE_TYPE_CDDB (that only work for freedb at
+  the moment - 18_10_06)
+
+\attention result is malloc'ed and must be freed by the caller after
+use.
+*/
 void mp3splt_write_freedb_file_result(splt_state *state, int disc_id,
     const char *cddb_file, int *error, int cddb_get_type,
     const char cddb_get_server[256], int port)
@@ -1054,6 +1279,16 @@ void mp3splt_write_freedb_file_result(splt_state *state, int disc_id,
   }
 }
 
+/*! Export our split points to a cue file
+
+  \param out_file The name of the file to output the split points to
+  \param state The splt_state structure containing the split points
+  \param error Contains the error code if anything goes wrong
+  \param stop_at_total_time If this parameter is !=0 we don't output
+  splitpoints that lie beyond the end of the audio data. Note that the
+  last splitpoint can be slightly beyond the calculated end of audio
+  data.
+*/
 void mp3splt_export_to_cue(splt_state *state, const char *out_file,
     short stop_at_total_time, int *error)
 {
@@ -1082,7 +1317,7 @@ void mp3splt_export_to_cue(splt_state *state, const char *out_file,
   }
 }
 
-//puts output format
+//! Sets the output format
 void mp3splt_set_oformat(splt_state *state,
     const char *format_string, int *error)
 {
@@ -1114,8 +1349,11 @@ void mp3splt_set_oformat(splt_state *state,
 /************************************/
 /* Other utilities                  */
 
-//returns the syncerrors found
-//puts error in the error variable
+/*! Determines if syncerrors are found
+
+  \param state The splt_state structure containing the split points
+  \param error The error code
+*/
 const splt_syncerrors *mp3splt_get_syncerrors(splt_state *state,
     int *error)
 {
@@ -1170,7 +1408,11 @@ const splt_syncerrors *mp3splt_get_syncerrors(splt_state *state,
   }
 }
 
-//puts possible error in the error variable
+/*! Split a file that has been generated by mp3wrap
+
+  \param state The splt_state structure containing the split points
+  \param error The error code
+*/
 const splt_wrap *mp3splt_get_wrap_files(splt_state *state,
     int *error)
 {
@@ -1222,7 +1464,7 @@ const splt_wrap *mp3splt_get_wrap_files(splt_state *state,
   }
 }
 
-//set the silence splitpoints without splitting
+//!set the silence splitpoints without actually splitting
 int mp3splt_set_silence_points(splt_state *state, int *error)
 {
   int erro = SPLT_OK;
@@ -1269,7 +1511,7 @@ int mp3splt_set_silence_points(splt_state *state, int *error)
   return found_splitpoints;
 }
 
-//count how many silence splitpoints we have with silence detection
+//!count how many silence splitpoints silence detection results in
 int mp3splt_count_silence_points(splt_state *state, int *error)
 {
   int number_of_tracks = mp3splt_set_silence_points(state, error) - 1;
@@ -1277,33 +1519,64 @@ int mp3splt_count_silence_points(splt_state *state, int *error)
   return number_of_tracks;
 }
 
-//returns libmp3splt version, max 20 chars
+//!returns libmp3splt version, max 20 chars
 void mp3splt_get_version(char *version)
 {
   snprintf(version,20,"%s",SPLT_PACKAGE_VERSION);
 }
 
-//result must be freed
+/*! Returns the error string that matches an error code
+
+\param state The splt_state The central structure containing all data
+for our library
+\param error The error code
+
+\attention The resulting string is malloc'ed and must be freed by the
+caller after use.
+*/
+
 char *mp3splt_get_strerror(splt_state *state, int error_code)
 {
   return splt_e_strerror(state, error_code);
 }
 
-//returns possible error or SPLT_OK if no error
+/*! Tell where to scan for plug-ins
+
+\param state The splt_state The central structure containing all data
+for our library
+\return The error code in case that anything goes wrong, else SPLT_OK
+*/
 int mp3splt_append_plugins_scan_dir(splt_state *state, char *dir)
 {
   return splt_p_append_plugin_scan_dir(state, dir);
 }
 
 #ifdef __WIN32__
-//returned result must be free'd
+/*! Converts a string from win32 to utf8
+
+\param The UTF16 string from Windows 
+\result The same string in UTF8
+\attention The resulting string is malloc'ed and must be freed by the
+caller after use.
+*/
 char *mp3splt_win32_utf16_to_utf8(const wchar_t *source)
 {
   return splt_w32_utf16_to_utf8(source);
 }
 #endif
 
-//returned result must be freed
+/*! TODO: What does this function do
+
+\param state The splt_state The central structure containing all data
+for our library
+\param num_of_files_found The number of files this library has found
+\param error The error code
+
+\attention The resulting string is malloc'ed and must be freed by the
+caller after use.
+
+\todo What does this function do?
+*/
 char **mp3splt_find_filenames(splt_state *state, const char *filename,
     int *num_of_files_found, int *error)
 {
@@ -1385,8 +1658,46 @@ char **mp3splt_find_filenames(splt_state *state, const char *filename,
   return found_files;
 }
 
+//@}
+
+//! Checks if a name points to a directory
 int mp3splt_u_check_if_directory(const char *fname)
 {
   return splt_io_check_if_directory(fname);
 }
 
+void mp3splt_free_one_tag(splt_tags *tags)
+{
+  splt_tu_free_one_tags(&tags);
+}
+
+splt_tags *mp3splt_parse_filename_regex(splt_state *state, int *error)
+{
+  splt_tags *tags = NULL;
+  int erro = SPLT_OK;
+  int *err = &erro;
+  if (error != NULL) { err = error; }
+
+  if (state != NULL)
+  {
+    if (!splt_o_library_locked(state))
+    {
+      splt_o_lock_library(state);
+
+      tags = splt_fr_parse_from_state(state, error);
+
+      splt_o_unlock_library(state);
+    }
+    else
+    {
+      *err = SPLT_ERROR_LIBRARY_LOCKED;
+    }
+  }
+  else
+  {
+    *err = SPLT_ERROR_STATE_NULL;
+  }
+
+  return tags;
+}
+
