@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -30,19 +30,56 @@
  *
  *********************************************************/
 
+/*! \file
+
+Automatic generation of filenams for split files from tags.
+ */
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
 
 #include "splt.h"
 
-static short splt_u_output_variable_is_valid(char v, int *amb);
-static const char *splt_u_get_format_ptr(const char *format, char *temp);
-static int splt_u_get_requested_num_of_digits(splt_state *state, const char *format,
-    int *requested_num_of_digits, int is_alpha);
-static void splt_u_alpha_track(splt_state *state, int nfield,
-    char *fm, int fm_length, int number_of_digits, int tracknumber);
-static char splt_of_get_number_of_digits_from_total_time(splt_state *state);
+/*! \brief Is a placeholder char valid in a filename format string?
+
+\param v The character that is to be tested
+\param amb True, if the character is valid
+\return The value that is returned in amb, as well.
+
+\todo Why do we need amb?
+ */
+static short splt_u_output_variable_is_valid(char v, int *amb)
+{
+  switch (v)
+  {
+    case 's':
+    case 'S':
+    case 'm':
+    case 'M':
+    case 'h':
+    case 'H':
+    case 'a':
+    case 'A':
+    case 'b':
+    case 'f':
+    case 'g':
+    case 'p':
+      break;
+    case 't':
+    case 'l':
+    case 'L':
+    case 'u':
+    case 'U':
+    case 'n':
+    case 'N':
+      *amb = SPLT_OUTPUT_FORMAT_OK;
+      break;
+    default:
+      return SPLT_FALSE;
+  }
+
+  return SPLT_TRUE;
+}
 
 int splt_of_parse_outformat(char *s, splt_state *state)
 {
@@ -153,6 +190,136 @@ int splt_of_parse_outformat(char *s, splt_state *state)
   return amb;
 }
 
+static const char *splt_u_get_format_ptr(const char *format, char *temp)
+{
+  int format_length = strlen(format);
+  const char *format_ptr = format;
+
+  if ((format_length > 2) && isdigit(format[2]))
+  {
+    temp[2] = format[2];
+    format_ptr = format + 1;
+  }
+
+  return format_ptr;
+}
+
+static int splt_u_get_requested_num_of_digits(splt_state *state, const char *format,
+    int *requested_num_of_digits, int is_alpha)
+{
+  int format_length = strlen(format);
+  int number_of_digits = 0;
+  if (is_alpha)
+  {
+    number_of_digits = state->oformat.output_alpha_format_digits;
+  }
+  else
+  {
+    number_of_digits = splt_of_get_oformat_number_of_digits_as_int(state);
+  }
+  int max_number_of_digits = number_of_digits;
+  *requested_num_of_digits = number_of_digits;
+
+  if ((format_length > 2) && isdigit(format[2]))
+  {
+    *requested_num_of_digits = format[2] - '0';
+  }
+
+  if (*requested_num_of_digits > number_of_digits)
+  {
+    max_number_of_digits = *requested_num_of_digits;
+  }
+
+  return max_number_of_digits;
+}
+
+/*! Encode track number as 'A', 'B', ... 'Z', 'AA, 'AB', ...
+ *
+ * This is not simply "base 26"; note that the lowest 'digit' is from 'A' to
+ * 'Z' (base 26), but all higher digits are 'A' to 'Z' plus 'nothing', i.e.,
+ * base 27. In other words, since after 'Z' comes 'AA', we cannot use 'AA'
+ * as the padded version of track number 1 ('A').
+ *
+ * This means that there are two distinct work modes:
+ * - The normal encoding is as described above.
+ * - When the user has specified the number of digits (padding), we encode
+ *   the track number as simple base-26: 'AAA', 'AAB', ... 'AAZ', 'ABA',
+ *   'ABB', ...
+ */
+static void splt_u_alpha_track(splt_state *state, int nfield,
+    char *fm, int fm_length, int number_of_digits, int tracknumber)
+{
+  char *format = state->oformat.format[nfield];
+  int lowercase = (toupper(format[1]) == 'L');
+  char a = lowercase ? 'a' : 'A';
+  int zerobased = tracknumber - 1;
+  int i = 1, min_digits = state->oformat.output_alpha_format_digits;
+
+  if (number_of_digits > 1)
+  {
+    /* Padding required => simple base-26 encoding */
+    if (number_of_digits < min_digits)
+      number_of_digits = min_digits;
+    for (i = 1; i <= number_of_digits; ++ i, zerobased /= 26)
+    {
+      int digit = (zerobased % 26);
+      fm[number_of_digits - i] = a + digit;
+    }
+  }
+  else
+  {
+    /* No padding: First letter base-26, others base-27 */
+    number_of_digits = min_digits;
+
+    /* Start with the first, base-26 'digit' */
+    fm[number_of_digits - 1] = a + (zerobased % 26);
+
+    /* Now handle all other digits */
+    zerobased /= 26;
+    for (i = 2; i <= number_of_digits; ++ i, zerobased /= 27)
+    {
+      int digit = (zerobased % 27);
+      fm[number_of_digits - i] = a + digit - 1;
+    }
+  }
+
+  int offset = 0;
+  if ((strlen(format) > 2) && isdigit(format[2]))
+  {
+    offset = 1;
+  }
+  snprintf(fm + number_of_digits, fm_length - number_of_digits,
+      "%s", format + 2 + offset);
+}
+
+static char splt_of_get_number_of_digits_from_total_time(splt_state *state)
+{
+  long total_time = splt_t_get_total_time(state);
+  if (total_time > 0)
+  {
+    long minutes = total_time / 100 / 60;
+    int i = (int) (log10l((long double) minutes));
+    char number_of_digits = (char) (i + '1');
+    if (number_of_digits == '1')
+    {
+      return '2';
+    }
+
+    return number_of_digits;
+  }
+
+  return '2';
+}
+
+/*! \brief Automagically set the filename for a split point
+
+  The filename is generated from the tags and the output format
+  string.
+
+  \param state The central structure libmp3splt keeps all its data in
+  \param current_splt The number of the split point to determine the
+  filename for.
+ */
 int splt_of_put_output_format_filename(splt_state *state, int current_split)
 {
   int error = SPLT_OK;
@@ -172,6 +339,7 @@ int splt_of_put_output_format_filename(splt_state *state, int current_split)
   char *title = NULL;
   char *artist = NULL;
   char *album = NULL;
+  char *genre = NULL;
   char *performer = NULL;
   char *artist_or_performer = NULL;
   char *original_filename = NULL;
@@ -489,6 +657,51 @@ put_value:
             snprintf(fm, fm_length, "%s", temp);
           }
           break;
+        case 'g':
+          if (splt_tu_tags_exists(state,tags_index))
+          {
+            //we get the genre
+            genre =
+              (char *)splt_tu_get_tags_field(state,tags_index, SPLT_TAGS_GENRE);
+            splt_su_clean_string(state, genre, &error);
+            if (error < 0) { goto end; };
+          }
+          else
+          {
+            genre = NULL;
+          }
+
+          //
+          if (genre != NULL)
+          {
+            int genre_length = 0;
+            genre_length = strlen(genre);
+            snprintf(temp+2, temp_len, state->oformat.format[i]+2);
+
+            fm_length = strlen(temp) + genre_length + 1;
+          }
+          else
+          {
+            snprintf(temp,temp_len, state->oformat.format[i]+2);
+            fm_length = strlen(temp) + 1;
+          }
+
+          if ((fm = malloc(fm_length * sizeof(char))) == NULL)
+          {
+            error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+            goto end;
+          }
+
+          //
+          if (genre != NULL)
+          {
+            snprintf(fm, fm_length, temp, genre);
+          }
+          else
+          {
+            snprintf(fm, fm_length, "%s", temp);
+          }
+          break;
         case 't':
           if (splt_tu_tags_exists(state,tags_index))
           {
@@ -593,6 +806,7 @@ put_value:
               ((split_mode != SPLT_OPTION_TIME_MODE) &&
                (split_mode != SPLT_OPTION_NORMAL_MODE) &&
                (split_mode != SPLT_OPTION_SILENCE_MODE) &&
+               (split_mode != SPLT_OPTION_TRIM_SILENCE_MODE) &&
                (split_mode != SPLT_OPTION_ERROR_MODE) &&
                (split_mode != SPLT_OPTION_LENGTH_MODE)))
           {
@@ -754,158 +968,3 @@ end:
   return error;
 }
 
-static char splt_of_get_number_of_digits_from_total_time(splt_state *state)
-{
-  long total_time = splt_t_get_total_time(state);
-  if (total_time > 0)
-  {
-    long minutes = total_time / 100 / 60;
-    int i = (int) (log10l((long double) minutes));
-    char number_of_digits = (char) (i + '1');
-    if (number_of_digits == '1')
-    {
-      return '2';
-    }
-
-    return number_of_digits;
-  }
-
-  return '2';
-}
-
-static short splt_u_output_variable_is_valid(char v, int *amb)
-{
-  switch (v)
-  {
-    case 's':
-    case 'S':
-    case 'm':
-    case 'M':
-    case 'h':
-    case 'H':
-    case 'a':
-    case 'A':
-    case 'b':
-    case 'f':
-    case 'p':
-      break;
-    case 't':
-    case 'l':
-    case 'L':
-    case 'u':
-    case 'U':
-    case 'n':
-    case 'N':
-      *amb = SPLT_OUTPUT_FORMAT_OK;
-      break;
-    default:
-      return SPLT_FALSE;
-  }
-
-  return SPLT_TRUE;
-}
-
-static const char *splt_u_get_format_ptr(const char *format, char *temp)
-{
-  int format_length = strlen(format);
-  const char *format_ptr = format;
-
-  if ((format_length > 2) && isdigit(format[2]))
-  {
-    temp[2] = format[2];
-    format_ptr = format + 1;
-  }
-
-  return format_ptr;
-}
-
-static int splt_u_get_requested_num_of_digits(splt_state *state, const char *format,
-    int *requested_num_of_digits, int is_alpha)
-{
-  int format_length = strlen(format);
-  int number_of_digits = 0;
-  if (is_alpha)
-  {
-    number_of_digits = state->oformat.output_alpha_format_digits;
-  }
-  else
-  {
-    number_of_digits = splt_of_get_oformat_number_of_digits_as_int(state);
-  }
-  int max_number_of_digits = number_of_digits;
-  *requested_num_of_digits = number_of_digits;
-
-  if ((format_length > 2) && isdigit(format[2]))
-  {
-    *requested_num_of_digits = format[2] - '0';
-  }
-
-  if (*requested_num_of_digits > number_of_digits)
-  {
-    max_number_of_digits = *requested_num_of_digits;
-  }
-
-  return max_number_of_digits;
-}
-
-/*
- * Encode track number as 'A', 'B', ... 'Z', 'AA, 'AB', ...
- *
- * This is not simply "base 26"; note that the lowest 'digit' is from 'A' to
- * 'Z' (base 26), but all higher digits are 'A' to 'Z' plus 'nothing', i.e.,
- * base 27. In other words, since after 'Z' comes 'AA', we cannot use 'AA'
- * as the padded version of track number 1 ('A').
- *
- * This means that there are two distinct work modes:
- * - The normal encoding is as described above.
- * - When the user has specified the number of digits (padding), we encode
- *   the track number as simple base-26: 'AAA', 'AAB', ... 'AAZ', 'ABA',
- *   'ABB', ...
- */
-static void splt_u_alpha_track(splt_state *state, int nfield,
-    char *fm, int fm_length, int number_of_digits, int tracknumber)
-{
-  char *format = state->oformat.format[nfield];
-  int lowercase = (toupper(format[1]) == 'L');
-  char a = lowercase ? 'a' : 'A';
-  int zerobased = tracknumber - 1;
-  int i = 1, min_digits = state->oformat.output_alpha_format_digits;
-
-  if (number_of_digits > 1)
-  {
-    /* Padding required => simple base-26 encoding */
-    if (number_of_digits < min_digits)
-      number_of_digits = min_digits;
-    for (i = 1; i <= number_of_digits; ++ i, zerobased /= 26)
-    {
-      int digit = (zerobased % 26);
-      fm[number_of_digits - i] = a + digit;
-    }
-  }
-  else
-  {
-    /* No padding: First letter base-26, others base-27 */
-    number_of_digits = min_digits;
-
-    /* Start with the first, base-26 'digit' */
-    fm[number_of_digits - 1] = a + (zerobased % 26);
-
-    /* Now handle all other digits */
-    zerobased /= 26;
-    for (i = 2; i <= number_of_digits; ++ i, zerobased /= 27)
-    {
-      int digit = (zerobased % 27);
-      fm[number_of_digits - i] = a + digit - 1;
-    }
-  }
-
-  int offset = 0;
-  if ((strlen(format) > 2) && isdigit(format[2]))
-  {
-    offset = 1;
-  }
-  snprintf(fm + number_of_digits, fm_length - number_of_digits,
-      "%s", format + 2 + offset);
-}
-
-
