@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -30,6 +30,11 @@
  *
  *********************************************************/
 
+/*! \file
+
+Actually split the input file
+*/
+
 #include <sys/stat.h>
 #include <string.h>
 #include <math.h>
@@ -37,8 +42,7 @@
 #include "splt.h"
 
 /****************************/
-/* splt normal split */
-
+/*! normal split */
 static long splt_s_real_split(double splt_beg, double splt_end, 
     int save_end_point, int *error, splt_state *state)
 {
@@ -70,6 +74,7 @@ static long splt_s_real_split(double splt_beg, double splt_end,
   return new_end_point;
 }
 
+//! Extract the file portion between two split points
 static long splt_s_split(splt_state *state, int first_splitpoint,
     int second_splitpoint, int *error)
 {
@@ -129,7 +134,7 @@ static long splt_s_split(splt_state *state, int first_splitpoint,
   return new_end_point;
 }
 
-//splits the file with multiple points
+//!splits the file with multiple points
 void splt_s_multiple_split(splt_state *state, int *error)
 {
   int split_type = splt_o_get_int_option(state, SPLT_OPT_SPLIT_MODE);
@@ -174,8 +179,11 @@ void splt_s_multiple_split(splt_state *state, int *error)
       err = splt_u_finish_tags_and_put_output_format_filename(state, i);
       if (err < 0) { *error = err; goto end; }
 
-      long new_end_point = splt_s_split(state, i, i+1, error);
-      splt_array_append(new_end_points, (void *)new_end_point);
+      int end_point_index = i+1;
+      long new_end_point = splt_s_split(state, i, end_point_index, error);
+      splt_pair *index_end_point =
+        splt_pair_new((void *) &end_point_index, (void *) &new_end_point);
+      splt_array_append(new_end_points, (void *)index_end_point);
 
       splt_sp_set_splitpoint_value(state, i + 1, saved_end_point);
 
@@ -196,8 +204,13 @@ void splt_s_multiple_split(splt_state *state, int *error)
 end:
   for (i = 0;i < splt_array_length(new_end_points);i++)
   {
-    splt_sp_set_splitpoint_value(state, i+1,
-        (long) splt_array_get(new_end_points, i));
+    splt_pair *index_end_point = (splt_pair *) splt_array_get(new_end_points, i);
+
+    splt_sp_set_splitpoint_value(state,
+        *((int*) splt_pair_first(index_end_point)),
+        *((long*) splt_pair_second(index_end_point)));
+
+    splt_pair_free(&index_end_point);
   }
 
   splt_array_free(&new_end_points);
@@ -215,7 +228,7 @@ void splt_s_normal_split(splt_state *state, int *error)
   splt_s_multiple_split(state, error);
 }
 
-//the sync error mode
+//!the sync error mode
 void splt_s_error_split(splt_state *state, int *error)
 {
   splt_c_put_info_message_to_client(state, _(" info: starting error mode split\n"));
@@ -336,7 +349,7 @@ bloc_end:
 }
 
 /************************************/
-/* splt time and length split */
+/*! time and length split           */
 
 static void splt_s_split_by_time(splt_state *state, int *error,
     double split_time_length, int number_of_files)
@@ -419,7 +432,11 @@ static void splt_s_split_by_time(splt_state *state, int *error,
           double new_sec_end_point = splt_p_split(state, final_fname,
               begin, overlapped_end, error, save_end_point);
           long new_end_point = splt_co_time_to_long_ceil(new_sec_end_point);
-          splt_array_append(new_end_points, (void *) new_end_point);
+
+          int end_point_index = current_split + 1;
+          splt_pair *index_end_point =
+            splt_pair_new((void *) &end_point_index, (void *) &new_end_point);
+          splt_array_append(new_end_points, (void *) index_end_point);
 
           //if no error for the split, put the split file
           if (*error >= 0)
@@ -475,9 +492,15 @@ static void splt_s_split_by_time(splt_state *state, int *error,
       int i = 0;
       for (i = 0;i < splt_array_length(new_end_points);i++)
       {
-        splt_sp_set_splitpoint_value(state, i+1,
-            (long) splt_array_get(new_end_points, i));
+        splt_pair *index_end_point = (splt_pair *) splt_array_get(new_end_points, i);
+
+        splt_sp_set_splitpoint_value(state,
+            *((int*) splt_pair_first(index_end_point)),
+            *((long*) splt_pair_second(index_end_point)));
+
+        splt_pair_free(&index_end_point);
       }
+
       splt_array_free(&new_end_points);
     }
     else
@@ -488,9 +511,10 @@ static void splt_s_split_by_time(splt_state *state, int *error,
     //we put the time split error
     switch (*error)
     {
-      case SPLT_MIGHT_BE_VBR: 
+/*      case SPLT_MIGHT_BE_VBR: 
         *error = SPLT_TIME_SPLIT_OK;
-        break;
+        break; */
+
       case SPLT_OK_SPLIT: 
         *error = SPLT_TIME_SPLIT_OK;
         break;
@@ -510,9 +534,11 @@ static void splt_s_split_by_time(splt_state *state, int *error,
   }
 }
 
-//function used with the -t option (time split
-//create an indefinite number of smaller files with a fixed time
-//length specified by options.split_time in seconds
+/*! function used with the -t option (time split)
+
+create an indefinite number of smaller files with a fixed time
+length specified by options.split_time in seconds
+*/
 void splt_s_time_split(splt_state *state, int *error)
 {
   splt_c_put_info_message_to_client(state, _(" info: starting time mode split\n"));
@@ -527,9 +553,11 @@ void splt_s_time_split(splt_state *state, int *error)
   splt_s_split_by_time(state, error, split_time_length, -1);
 }
 
-//function used with the -L option (length split
-//split into X files
-//X is defined by SPLT_OPT_LENGTH_SPLIT_FILE_NUMBER
+/*! function used with the -L option (length split)
+
+split into X files
+X is defined by SPLT_OPT_LENGTH_SPLIT_FILE_NUMBER
+*/
 void splt_s_equal_length_split(splt_state *state, int *error)
 {
   splt_c_put_info_message_to_client(state, _(" info: starting 'split in equal tracks' mode\n"));
@@ -563,12 +591,75 @@ void splt_s_equal_length_split(splt_state *state, int *error)
   }
 }
 
-/************************************/
-/* splt silence detection and split */
+int splt_s_set_trim_silence_splitpoints(splt_state *state, int *error)
+{
+  splt_d_print_debug(state, "Search and set trim silence splitpoints...\n");
 
-//returns the number of silence splits found
-//or the number of tracks specified in the options
-//sets the silence splitpoints in state->split.splitpoints
+  int found = 0;
+  struct splt_ssplit *temp = NULL;
+  int append_error = SPLT_OK;
+
+  if (! splt_o_get_int_option(state,SPLT_OPT_QUIET_MODE))
+  {
+    splt_c_put_info_message_to_client(state, 
+        _(" Trim silence split - Th: %.1f dB\n"),
+        splt_o_get_float_option(state, SPLT_OPT_PARAM_THRESHOLD));
+  }
+
+  if (state->split.get_silence_level)
+  {
+    state->split.get_silence_level(0, INT_MAX, state->split.silence_level_client_data);
+  }
+  found = splt_p_scan_trim_silence(state, error);
+  if (*error < 0) { goto end; }
+
+  if (splt_t_split_is_canceled(state))
+  {
+    *error = SPLT_SPLIT_CANCELLED;
+    goto end;
+  }
+
+  temp = state->silence_list;
+  int i;
+  long previous = 0;
+  for (i = 1; i < found + 1; i++)
+  {
+    if (temp == NULL)
+    {
+      found = i;
+      break;
+    }
+
+    long temp_silence_pos = splt_siu_silence_position(temp, 0) * 100;
+
+    if (i > 1 && temp_silence_pos < previous)
+    {
+      temp_silence_pos = LONG_MAX;
+    }
+
+    append_error = splt_sp_append_splitpoint(state, temp_silence_pos, NULL, SPLT_SPLITPOINT);
+    if (append_error != SPLT_OK) { *error = append_error; found = i; break; }
+
+    temp = temp->next;
+
+    previous = temp_silence_pos;
+  }
+
+end:
+  splt_siu_ssplit_free(&state->silence_list);
+  splt_t_set_splitnumber(state, found);
+
+  return found;
+}
+
+/************************************/
+/*! Split with split points setermined by silence detection
+
+Sets the silence splitpoints in state->split.splitpoints
+
+\return the number of split points found or the number of tracks
+specified in the options  
+*/
 int splt_s_set_silence_splitpoints(splt_state *state, int *error)
 {
   splt_d_print_debug(state,"Search and set silence splitpoints...\n");
@@ -667,6 +758,7 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
         remove_str);
   }
  
+  short read_silence_from_logs = SPLT_FALSE;
   if (we_read_silence_from_logs)
   {
     if (state->split.get_silence_level)
@@ -676,7 +768,7 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
 
     splt_c_put_info_message_to_client(state, 
         _(" Found silence log file '%s' ! Reading"
-          " silence points from file to save time ;)"), log_fname);
+          " silence points from file to save time ;)\n"), log_fname);
 
     found = splt_siu_parse_ssplit_file(state, log_file, error);
     if (log_file)
@@ -684,12 +776,13 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
       fclose(log_file);
       log_file = NULL;
     }
+
+    read_silence_from_logs = SPLT_TRUE;
   }
   else
   {
     if (state->split.get_silence_level)
     {
-      //TODO
       state->split.get_silence_level(0, INT_MAX, state->split.silence_level_client_data);
     }
     found = splt_p_scan_silence(state, error);
@@ -698,8 +791,12 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
   //if no error
   if (*error >= 0)
   {
-    splt_c_put_info_message_to_client(state,
-        _("\n Total silence points found: %d."),found);
+    if (!read_silence_from_logs)
+    {
+      splt_c_put_info_message_to_client(state, "\n");
+    }
+
+    splt_c_put_info_message_to_client(state, _(" Total silence points found: %d."), found);
 
     if (found > 0)
     {
@@ -758,12 +855,11 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
             if (append_error < 0) { *error = append_error; found = i; break;}
             append_error = splt_sp_append_splitpoint(state, 0, NULL, SPLT_SPLITPOINT);
             if (append_error < 0) { *error = append_error; found = i; break;}
-            splt_sp_set_splitpoint_value(state, 2*i-1,splt_co_time_to_long(temp->begin_position));
+            splt_sp_set_splitpoint_value(state, 2*i-1, splt_co_time_to_long(temp->begin_position));
             splt_sp_set_splitpoint_value(state, 2*i, splt_co_time_to_long(temp->end_position));
           }
           else
           {
-            //TODO
             long temp_silence_pos = splt_siu_silence_position(temp, offset) * 100;
             append_error = splt_sp_append_splitpoint(state, temp_silence_pos, NULL, SPLT_SPLITPOINT);
             if (append_error != SPLT_OK) { *error = append_error; found = i; break; }
@@ -846,7 +942,6 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
     }
   }
 
-
   splt_siu_ssplit_free(&state->silence_list);
 
   splt_t_set_splitnumber(state, splitpoints_appended + 1);
@@ -854,8 +949,10 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
   return found;
 }
 
-//do the silence split
-//possible error in error
+/*! Do the silence split
+
+\param error The code of a eventual error that has occoured
+*/ 
 void splt_s_silence_split(splt_state *state, int *error)
 {
   splt_d_print_debug(state,"Starting silence split ...\n");
@@ -863,8 +960,7 @@ void splt_s_silence_split(splt_state *state, int *error)
   //print some useful infos to the client
   splt_c_put_info_message_to_client(state, _(" info: starting silence mode split\n"));
 
-  int found = 0;
-  found = splt_s_set_silence_splitpoints(state, error);
+  int found = splt_s_set_silence_splitpoints(state, error);
 
   //if no error
   if (*error >= 0)
@@ -887,9 +983,6 @@ void splt_s_silence_split(splt_state *state, int *error)
       //we put the silence split errors
       switch (*error)
       {
-        case SPLT_MIGHT_BE_VBR:
-          *error = SPLT_SILENCE_OK;
-          break;
         case SPLT_OK_SPLIT:
           *error = SPLT_SILENCE_OK;
           break;
@@ -907,10 +1000,52 @@ void splt_s_silence_split(splt_state *state, int *error)
   }
 }
 
-/****************************/
-/* splt wrap split */
+/*! Do the trim silence split
 
-//do the wrap split
+\param error The code of a eventual error that has occoured
+*/ 
+void splt_s_trim_silence_split(splt_state *state, int *error)
+{
+  splt_c_put_info_message_to_client(state, _(" info: starting trim using silence mode split\n"));
+
+  int found = splt_s_set_trim_silence_splitpoints(state, error);
+  if (*error < 0) { return; }
+
+  if (found < 1)
+  {
+    *error = SPLT_NO_SILENCE_SPLITPOINTS_FOUND;
+    return;
+  }
+
+  splt_d_print_debug(state,"Writing tracks...\n");
+
+  int output_filenames = splt_o_get_int_option(state,SPLT_OPT_OUTPUT_FILENAMES);
+  if (output_filenames == SPLT_OUTPUT_DEFAULT)
+  {
+    splt_of_set_oformat(state, SPLT_DEFAULT_TRIM_SILENCE_OUTPUT, error, SPLT_TRUE);
+    if (*error < 0) { return; }
+  }
+
+  splt_s_multiple_split(state, error);
+
+  switch (*error)
+  {
+    case SPLT_OK_SPLIT:
+      *error = SPLT_TRIM_SILENCE_OK;
+      break;
+    case SPLT_OK_SPLIT_EOF:
+      *error = SPLT_TRIM_SILENCE_OK;
+      break;
+    default:
+      break;
+  }
+}
+
+/****************************/
+/*! Automatically split a file that has been created by mp3wrap
+
+do the wrap split
+*/
 void splt_s_wrap_split(splt_state *state, int *error)
 {
   char *new_filename_path = splt_t_get_new_filename_path(state);
