@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -30,6 +30,10 @@
  *
  *********************************************************/
 
+/*! \file
+
+String utilities
+*/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,11 +45,22 @@
 
 #include "splt.h"
 
-static int splt_su_append_one(char **str, const char *to_append,
-    size_t to_append_size);
-static void splt_su_clean_string_(splt_state *state, char *s, int *error,
-    int ignore_dirchar);
-static int splt_su_is_illegal_char(char c, int ignore_dirchar);
+void splt_su_replace_all_char(char *str, char to_replace, char replacement)
+{
+  if (str == NULL)
+  {
+    return;
+  }
+
+  int i = 0;
+  for (i = 0;i < strlen(str);i++)
+  {
+    if (str[i] == to_replace)
+    {
+      str[i] = replacement;
+    }
+  }
+}
 
 char *splt_su_replace_all(const char *str, char *to_replace,
     char *replacement, int *error)
@@ -114,6 +129,41 @@ int splt_su_append_str(char **str, const char *to_append, ...)
   return err;
 }
 
+static int splt_su_append_one(char **str, const char *to_append, size_t to_append_size)
+{
+  if (str == NULL || to_append == NULL || to_append[0] == '\0' || to_append_size == 0)
+  {
+    return SPLT_OK;
+  }
+
+  size_t new_size = 0;
+
+  if (*str == NULL)
+  {
+    new_size = to_append_size + 1;
+    *str = malloc(new_size);
+    if (*str == NULL)
+    {
+      return SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    }
+
+    *str[0] = '\0';
+  }
+  else
+  {
+    new_size = to_append_size + strlen(*str) + 1;
+    *str = realloc(*str, new_size);
+    if (*str == NULL)
+    {
+      return SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    }
+  }
+
+  strncat(*str, to_append, to_append_size);
+
+  return SPLT_OK;
+}
+
 int splt_su_append(char **str, const char *to_append, ...)
 {
   int err = SPLT_OK;
@@ -134,6 +184,37 @@ int splt_su_append(char **str, const char *to_append, ...)
   return err;
 }
 
+int splt_su_set(char **str, const char *to_append, ...)
+{
+  if (!str)
+  {
+    return SPLT_OK;
+  }
+
+  if (*str)
+  {
+    free(*str);
+    *str = NULL;
+  }
+
+  int err = SPLT_OK;
+  va_list ap;
+
+  va_start(ap, to_append);
+
+  while (to_append)
+  {
+    size_t to_append_size = va_arg(ap, size_t);
+    err = splt_su_append_one(str, to_append, to_append_size);
+    if (err < 0) { break; }
+    to_append = va_arg(ap, const char *);
+  }
+
+  va_end(ap);
+
+  return err;
+}
+
 void splt_su_free_replace(char **str, char *replacement)
 {
   if (!str)
@@ -179,6 +260,68 @@ int splt_su_copy(const char *src, char **dest)
   return SPLT_OK;
 }
 
+static int splt_su_is_illegal_char(char c, int ignore_dirchar)
+{
+  if ((ignore_dirchar) && (c == SPLT_DIRCHAR))
+  {
+    return SPLT_FALSE;
+  }
+
+  //for the sake of filename portability, we take the the windows illegal
+  //characters (will be changed upon feature request)
+  if ((c == '\\') || (c == '/') || (c == ':') || (c == '*') ||
+      (c == '?') || (c == '"') || (c == '<') ||
+      (c == '>') || (c == '|') || (c == '\r'))
+  {
+    return SPLT_TRUE;
+  }
+
+  return SPLT_FALSE;
+}
+
+static void splt_su_clean_string_(splt_state *state, char *s, int *error, int ignore_dirchar)
+{
+  int i = 0, j=0;
+  char *copy = NULL;
+  if (s)
+  {
+    copy = strdup(s);
+    if (copy)
+    {
+      for (i=0; i<=strlen(copy); i++)
+      {
+        if (! splt_su_is_illegal_char(copy[i], ignore_dirchar))
+        {
+          s[j++] = copy[i];
+        }
+        else
+        {
+          s[j++] = '_';
+        }
+      }
+      free(copy);
+      copy = NULL;
+
+      // Trim string. I will never stop to be surprised about cddb strings dirtiness! ;-)
+      for (i=strlen(s)-1; i >= 0; i--) 
+      {
+        if (s[i]==' ')
+        {
+          s[i] = '\0';
+        }
+        else 
+        {
+          break;
+        }
+      }
+    }
+    else
+    {
+      *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    }
+  }
+}
+
 void splt_su_clean_string(splt_state *state, char *s, int *error)
 {
   splt_su_clean_string_(state, s, error, SPLT_FALSE);
@@ -194,8 +337,23 @@ char *splt_su_cut_spaces(char *c)
   return c;
 }
 
+const char *splt_su_skip_spaces(const char *c)
+{
+  while (*c == ' ' || *c == '\t')
+  {
+    c++;
+  }
+
+  return c;
+}
+
 void splt_su_cut_spaces_from_end(char *c)
 {
+  if (c == NULL || *c == '\0')
+  {
+    return;
+  }
+
   char *end = strchr(c, '\0');
   if (!end)
   {
@@ -217,7 +375,7 @@ char *splt_su_trim_spaces(char *c)
   return splt_su_cut_spaces(c);
 }
 
-int splt_su_is_empty_line(char *line)
+int splt_su_is_empty_line(const char *line)
 {
   if (!line)
   {
@@ -250,6 +408,29 @@ void splt_su_line_to_unix(char *line)
   }
 }
 
+void splt_su_keep_path_and_remove_filename(char *path)
+{
+  char *last_dirchar = strrchr(path, SPLT_DIRCHAR);
+  if (last_dirchar == NULL)
+  {
+    return;
+  }
+
+  *(last_dirchar+1) = '\0';
+
+#ifdef __WIN32__
+  if (!splt_w32_str_is_drive_root_directory(path))
+  {
+    *last_dirchar = '\0';
+  }
+#else
+  if (last_dirchar != path)
+  {
+    *last_dirchar = '\0';
+  }
+#endif
+}
+
 const char *splt_su_get_fname_without_path(const char *filename)
 {
   char *c = NULL;
@@ -261,6 +442,22 @@ const char *splt_su_get_fname_without_path(const char *filename)
   return filename;
 }
 
+char *splt_su_get_fname_without_path_and_extension(const char *filename, int *error)
+{
+  const char *fname_without_path = splt_su_get_fname_without_path(filename);
+
+  char *fname_without_path_and_extension = NULL;
+  int err = splt_su_copy(fname_without_path, &fname_without_path_and_extension);
+  if (err < 0) {
+    *error = err;
+    return NULL;
+  }
+
+  splt_su_cut_extension(fname_without_path_and_extension);
+
+  return fname_without_path_and_extension;
+}
+
 char *splt_su_get_fname_with_path_and_extension(splt_state *state, int *error)
 {
   int err = SPLT_OK;
@@ -346,32 +543,6 @@ void splt_su_cut_extension(char *str)
   }
 }
 
-char *splt_su_str_to_upper(const char *str, int *error)
-{
-  int err = SPLT_OK;
-
-  if (!str)
-  {
-    return NULL;
-  }
-
-  char *result = NULL;
-  err = splt_su_copy(str, &result);
-  if (err < 0)
-  {
-    *error = err;
-    return NULL;
-  }
-
-  int i = 0;
-  for (i = 0;i < strlen(str);i++)
-  {
-    result[i] = toupper(str[i]);
-  }
-
-  return result;
-}
-
 void splt_su_str_cut_last_char(char *str)
 {
   if (!str)
@@ -508,100 +679,106 @@ char *splt_su_get_formatted_message(splt_state *state, char *message, ...)
   return mess;
 }
 
-static int splt_su_append_one(char **str, const char *to_append, size_t to_append_size)
+int splt_su_str_line_has_digit(const char *str)
 {
-  if (str == NULL || to_append == NULL || to_append[0] == '\0' || to_append_size == 0)
+  while (*str != '\0')
   {
-    return SPLT_OK;
-  }
-
-  size_t new_size = 0;
-
-  if (*str == NULL)
-  {
-    new_size = to_append_size + 1;
-    *str = malloc(new_size);
-    if (*str == NULL)
+    if (isdigit(*str))
     {
-      return SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+      return SPLT_TRUE;
     }
 
-    *str[0] = '\0';
-  }
-  else
-  {
-    new_size = to_append_size + strlen(*str) + 1;
-    *str = realloc(*str, new_size);
-    if (*str == NULL)
-    {
-      return SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-    }
-  }
-
-  strncat(*str, to_append, to_append_size);
-
-  return SPLT_OK;
-}
-
-static int splt_su_is_illegal_char(char c, int ignore_dirchar)
-{
-  if ((ignore_dirchar) && (c == SPLT_DIRCHAR))
-  {
-    return SPLT_FALSE;
-  }
-
-  //for the sake of filename portability, we take the the windows illegal
-  //characters (will be changed upon feature request)
-  if ((c == '\\') || (c == '/') || (c == ':') || (c == '*') ||
-      (c == '?') || (c == '"') || (c == '<') ||
-      (c == '>') || (c == '|') || (c == '\r'))
-  {
-    return SPLT_TRUE;
+    str++;
   }
 
   return SPLT_FALSE;
 }
 
-static void splt_su_clean_string_(splt_state *state, char *s, int *error, int ignore_dirchar)
+static char *splt_su_str_to_func(const char *str, int (*conversion_func)(int), int *error)
 {
-  int i = 0, j=0;
-  char *copy = NULL;
-  if (s)
+  int err = SPLT_OK;
+
+  if (!str)
   {
-    copy = strdup(s);
-    if (copy)
+    return NULL;
+  }
+
+  char *result = NULL;
+  err = splt_su_copy(str, &result);
+  if (err < 0)
+  {
+    *error = err;
+    return NULL;
+  }
+
+  int i = 0;
+  for (i = 0;i < strlen(str);i++)
+  {
+    result[i] = conversion_func(str[i]);
+  }
+
+  return result;
+}
+
+
+char *splt_su_convert(const char *str, splt_str_format format, int *error)
+{
+  if (str == NULL)
+  {
+    return NULL;
+  }
+
+  char *new_str = NULL;
+
+  int lastspace = 1;
+  int i = 0;
+
+  if (format != SPLT_TO_LOWERCASE && format != SPLT_TO_UPPERCASE)
+  {
+    int err = splt_su_copy(str, &new_str);
+    if (err < 0)
     {
-      for (i=0; i<=strlen(copy); i++)
+      *error = err;
+      return NULL;
+    }
+  }
+
+  switch (format)
+  {
+    case SPLT_NO_CONVERSION:
+      return new_str;
+      break;
+    case SPLT_TO_LOWERCASE:
+      return splt_su_str_to_func(str, tolower, error);
+      break;
+    case SPLT_TO_UPPERCASE:
+      return splt_su_str_to_func(str, toupper, error);
+      break;
+    case SPLT_TO_FIRST_UPPERCASE:
+      new_str[0] = toupper(new_str[0]);
+      return new_str;
+      break;
+    case SPLT_TO_WORD_FIRST_UPPERCASE:
+      for (i = 0; i < strlen(new_str); i++)
       {
-        if (! splt_su_is_illegal_char(copy[i], ignore_dirchar))
+        if (lastspace && new_str[i] != ' ')
         {
-          s[j++] = copy[i];
+          new_str[i] = toupper(new_str[i]);
+        }
+
+        if (new_str[i] == ' ')
+        {
+          lastspace = 1;
         }
         else
         {
-          s[j++] = '_';
+          lastspace = 0;
         }
       }
-      free(copy);
-      copy = NULL;
-
-      // Trim string. I will never stop to be surprised about cddb strings dirtiness! ;-)
-      for (i=strlen(s)-1; i >= 0; i--) 
-      {
-        if (s[i]==' ')
-        {
-          s[i] = '\0';
-        }
-        else 
-        {
-          break;
-        }
-      }
-    }
-    else
-    {
-      *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-    }
+      return new_str;
+      break;
   }
+
+  return NULL;
 }
 
