@@ -3,7 +3,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  *********************************************************/
 
@@ -30,44 +30,51 @@
 #include "splt.h"
 #include "tags_utils.h"
 
-static char *splt_tu_get_replaced_with_tags(const char *word,
-    const splt_tags *tags, int track, int *err, int replace_tags_in_tags);
-static splt_tags *splt_tu_get_tags_to_replace_in_tags(splt_state *state);
-static int splt_tu_set_on_tags_field(splt_tags *tags,
-    int tags_field, const void *data);
-static void splt_tu_set_empty_tags(splt_state *state, int index);
-static void splt_tu_free_one_tags(splt_tags *tags);
-
 void splt_tu_free_original_tags(splt_state *state)
 {
-  if (state->original_tags.year)
+  splt_tags *tags = &state->original_tags.tags;
+
+  if (tags->year)
   {
-    free(state->original_tags.year);
-    state->original_tags.year = NULL;
+    free(tags->year);
+    tags->year = NULL;
   }
-  if (state->original_tags.artist)
+  if (tags->artist)
   {
-    free(state->original_tags.artist);
-    state->original_tags.artist = NULL;
+    free(tags->artist);
+    tags->artist = NULL;
   }
-  if (state->original_tags.album)
+  if (tags->album)
   {
-    free(state->original_tags.album);
-    state->original_tags.album = NULL;
+    free(tags->album);
+    tags->album = NULL;
   }
-  if (state->original_tags.title)
+  if (tags->title)
   {
-    free(state->original_tags.title);
-    state->original_tags.title = NULL;
+    free(tags->title);
+    tags->title = NULL;
   }
-  if (state->original_tags.comment)
+  if (tags->comment)
   {
-    free(state->original_tags.comment);
-    state->original_tags.comment = NULL;
+    free(tags->comment);
+    tags->comment = NULL;
   }
-  state->original_tags.track = -INT_MAX;
-  //12 means "other"
-  state->original_tags.genre = 12;
+  if (tags->genre)
+  {
+    free(tags->genre);
+    tags->genre = NULL;
+  }
+  tags->track = -1;
+
+  int err = SPLT_OK;
+  splt_p_clear_original_tags(state, &err);
+}
+
+//! Get all tags for split point number x
+splt_tags *splt_tu_get_tags(splt_state *state, int *tags_number)
+{
+  *tags_number = state->split.real_tagsnumber;
+  return state->split.tags;
 }
 
 void splt_tu_auto_increment_tracknumber(splt_state *state)
@@ -129,6 +136,51 @@ void splt_tu_get_original_tags(splt_state *state, int *err)
   }
 }
 
+int splt_tu_append_tags(splt_state *state, 
+    const char *title, const char *artist,
+    const char *album, const char *performer,
+    const char *year, const char *comment,
+    int track, const char *genre, int set_original_tags)
+{
+  int error = SPLT_OK;
+  int old_tagsnumber = state->split.real_tagsnumber;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_TITLE, title);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_ARTIST, artist);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_ALBUM, album);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_PERFORMER, performer);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_YEAR, year);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_COMMENT, comment);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_TRACK, &track);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_ORIGINAL, &set_original_tags);
+  if (error != SPLT_OK)
+    return error;
+
+  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_GENRE, genre);
+  return error;
+}
+
 int splt_tu_append_original_tags(splt_state *state)
 {
   int err = SPLT_OK;
@@ -138,25 +190,30 @@ int splt_tu_append_original_tags(splt_state *state)
   char *new_album = NULL;
   char *new_year = NULL;
   char *new_comment = NULL;
+  char *new_genre = NULL;
 
-  new_title = splt_su_replace_all(state->original_tags.title, "@", "@@", &err);
+  splt_tags *tags = &state->original_tags.tags;
+
+  new_title = splt_su_replace_all(tags->title, "@", "@@", &err);
   if (err != SPLT_OK) { goto end; }
 
-  new_artist = splt_su_replace_all(state->original_tags.artist, "@", "@@", &err);
+  new_artist = splt_su_replace_all(tags->artist, "@", "@@", &err);
   if (err != SPLT_OK) { goto end; }
 
-  new_album = splt_su_replace_all(state->original_tags.album, "@", "@@", &err);
+  new_album = splt_su_replace_all(tags->album, "@", "@@", &err);
   if (err != SPLT_OK) { goto end; }
 
-  new_year = splt_su_replace_all(state->original_tags.year, "@", "@@", &err);
+  new_year = splt_su_replace_all(tags->year, "@", "@@", &err);
   if (err != SPLT_OK) { goto end; }
 
-  new_comment = splt_su_replace_all(state->original_tags.comment, "@", "@@", &err);
+  new_comment = splt_su_replace_all(tags->comment, "@", "@@", &err);
+  if (err != SPLT_OK) { goto end; }
+
+  new_genre = splt_su_replace_all(tags->genre, "@", "@@", &err);
   if (err != SPLT_OK) { goto end; }
 
   err = splt_tu_append_tags(state, new_title, new_artist, new_album, NULL,
-      new_year, new_comment, state->original_tags.track,
-      state->original_tags.genre);
+      new_year, new_comment, tags->track, new_genre, SPLT_TRUE);
 
 end:
   if (new_title) { free(new_title); }
@@ -164,64 +221,16 @@ end:
   if (new_album) { free(new_album); }
   if (new_year) { free(new_year); }
   if (new_comment) { free(new_comment); }
+  if (new_genre) { free(new_genre); }
 
   return err;
 }
 
-int splt_tu_append_tags(splt_state *state, 
-    const char *title, const char *artist,
-    const char *album, const char *performer,
-    const char *year, const char *comment,
-    int track, unsigned char genre)
-{
-  int error = SPLT_OK;
-  int old_tagsnumber = state->split.real_tagsnumber;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_TITLE, title);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_ARTIST, artist);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_ALBUM, album);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_PERFORMER, performer);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_YEAR, year);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_COMMENT, comment);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state,
-      old_tagsnumber, SPLT_TAGS_TRACK, &track);
-  if (error != SPLT_OK)
-    return error;
-
-  error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_GENRE, &genre);
-  return error;
-}
-
-//only if char non null and track != -1 and genre != 12
 int splt_tu_append_only_non_null_previous_tags(splt_state *state, 
     const char *title, const char *artist,
     const char *album, const char *performer,
     const char *year, const char *comment,
-    int track, unsigned char genre)
+    int track, const char *genre, int set_original_tags)
 {
   int error = SPLT_OK;
   int old_tagsnumber = state->split.real_tagsnumber-1;
@@ -230,56 +239,63 @@ int splt_tu_append_only_non_null_previous_tags(splt_state *state,
   {
     if (title != NULL)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_TITLE, title);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_TITLE, title);
     }
     if (error != SPLT_OK)
       return error;
+
     if (artist != NULL)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_ARTIST, artist);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_ARTIST, artist);
     }
     if (error != SPLT_OK)
       return error;
+
     if (album != NULL)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_ALBUM, album);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_ALBUM, album);
     }
     if (error != SPLT_OK)
       return error;
+
     if (performer != NULL)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_PERFORMER, performer);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_PERFORMER, performer);
     }
     if (error != SPLT_OK)
       return error;
+
     if (year != NULL)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_YEAR, year);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_YEAR, year);
     }
     if (error != SPLT_OK)
       return error;
+
     if (comment != NULL)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_COMMENT, comment);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_COMMENT, comment);
     }
     if (error != SPLT_OK)
       return error;
+
     if (track != -1)
     {
-      error = splt_tu_set_tags_field(state,
-          old_tagsnumber, SPLT_TAGS_TRACK, &track);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_TRACK, &track);
     }
     if (error != SPLT_OK)
       return error;
-    if (genre != 12)
+
+    if (set_original_tags != -1)
     {
-      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_GENRE, &genre);
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_ORIGINAL, &set_original_tags);
+    }
+    if (error != SPLT_OK)
+      return error;
+
+    if (genre != NULL)
+    {
+      error = splt_tu_set_tags_field(state, old_tagsnumber, SPLT_TAGS_GENRE, genre);
     }
   }
 
@@ -294,9 +310,32 @@ void splt_tu_reset_tags(splt_tags *tags)
   tags->performer = NULL;
   tags->year = NULL;
   tags->comment = NULL;
-  tags->track = -INT_MAX;
-  tags->genre = 0x0;
+  tags->track = -1;
+  tags->genre = NULL;
   tags->tags_version = 0;
+  tags->set_original_tags = SPLT_FALSE;
+}
+
+splt_tags *splt_tu_new_tags(splt_state *state, int *error)
+{
+  splt_tags *tags = malloc(sizeof(splt_tags));
+
+  if (tags == NULL)
+  {
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    return NULL;
+  }
+
+  memset(tags, sizeof(splt_tags), '\0');
+
+  splt_tu_reset_tags(tags);
+
+  return tags;
+}
+
+static void splt_tu_set_empty_tags(splt_state *state, int index)
+{
+  splt_tu_reset_tags(&state->split.tags[index]);
 }
 
 int splt_tu_new_tags_if_necessary(splt_state *state, int index)
@@ -311,9 +350,9 @@ int splt_tu_new_tags_if_necessary(splt_state *state, int index)
     }
     else
     {
-      if ((state->split.tags = malloc(sizeof(splt_tags))) == NULL)
+      state->split.tags = splt_tu_new_tags(state, &error);
+      if (error < 0)
       {
-        error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
         return error;
       }
       else
@@ -379,7 +418,7 @@ int splt_tu_set_tags_field(splt_state *state, int index,
   }
   else
   {
-    splt_tu_set_on_tags_field(&state->split.tags[index], tags_field, data);
+    splt_tu_set_field_on_tags(&state->split.tags[index], tags_field, data);
   }
 
   if (error != SPLT_OK)
@@ -392,271 +431,22 @@ int splt_tu_set_tags_field(splt_state *state, int index,
 
 int splt_tu_set_like_x_tags_field(splt_state *state, int tags_field, const void *data)
 {
-  return splt_tu_set_on_tags_field(&state->split.tags_like_x, tags_field, data);
+  return splt_tu_set_field_on_tags(&state->split.tags_like_x, tags_field, data);
 }
 
 int splt_tu_set_original_tags_field(splt_state *state, int tags_field, const void *data)
 {
-  return splt_tu_set_on_tags_field(&state->original_tags, tags_field, data);
+  return splt_tu_set_field_on_tags(&state->original_tags.tags, tags_field, data);
 }
 
-int splt_tu_set_tags_in_tags(splt_state *state, int current_split)
+void splt_tu_set_original_tags_data(splt_state *state, void *data)
 {
-  int err = SPLT_OK;
-
-  splt_tags *tags = splt_tu_get_tags_to_replace_in_tags(state);
-
-  if (tags)
-  {
-    int track = 0;
-    if (tags->track > 0)
-    {
-      track = tags->track;
-    }
-    else
-    {
-      if (current_split != -1)
-      {
-        track = current_split + 1;
-      }
-      else
-      {
-        track = splt_t_get_current_split_file_number(state);
-      }
-    }
-
-    splt_tags *cur_tags = splt_tu_get_current_tags(state);
-    cur_tags->track = track;
-    cur_tags->genre = tags->genre;
-    cur_tags->tags_version = tags->tags_version;
-
-    int replace_tags_in_tags = splt_o_get_int_option(state, SPLT_OPT_REPLACE_TAGS_IN_TAGS);
-
-    char *t = splt_tu_get_replaced_with_tags(tags->title, tags, track, &err, replace_tags_in_tags);
-    if (err != SPLT_OK) { return err; }
-    char *y = splt_tu_get_replaced_with_tags(tags->year, tags, track, &err, replace_tags_in_tags);
-    if (err != SPLT_OK) { return err; }
-    char *a = splt_tu_get_replaced_with_tags(tags->artist, tags, track, &err, replace_tags_in_tags);
-    if (err != SPLT_OK) { return err; }
-    char *al = splt_tu_get_replaced_with_tags(tags->album, tags, track, &err, replace_tags_in_tags);
-    if (err != SPLT_OK) { return err; }
-    char *c = splt_tu_get_replaced_with_tags(tags->comment, tags, track, &err, replace_tags_in_tags);
-    if (err != SPLT_OK) { return err; }
-
-    splt_su_free_replace(&cur_tags->title, t);
-    splt_su_free_replace(&cur_tags->year, y);
-    splt_su_free_replace(&cur_tags->artist, a);
-    splt_su_free_replace(&cur_tags->album, al);
-    splt_su_free_replace(&cur_tags->comment, c);
-  }
-
-  return err;
+  state->original_tags.all_original_tags = data;
 }
 
-splt_tags *splt_tu_get_tags(splt_state *state, int *tags_number)
+void *splt_tu_get_original_tags_data(splt_state *state)
 {
-  *tags_number = state->split.real_tagsnumber;
-  return state->split.tags;
-}
-
-splt_tags *splt_tu_get_tags_at(splt_state *state, int tags_index)
-{
-  if (!splt_tu_tags_exists(state, tags_index))
-  {
-    return NULL;
-  }
-
-  return &state->split.tags[tags_index];
-}
-
-splt_tags splt_tu_get_last_tags(splt_state *state)
-{
-  return state->split.tags[state->split.real_tagsnumber-1];
-}
-
-void *splt_tu_get_tags_field(splt_state *state, int index, int tags_field)
-{
-  if ((index >= state->split.real_tagsnumber) || (index < 0))
-  {
-    splt_e_error(SPLT_IERROR_INT,__func__, index, NULL);
-    return NULL;
-  }
-  else
-  {
-    switch(tags_field)
-    {
-      case SPLT_TAGS_TITLE:
-        return state->split.tags[index].title;
-        break;
-      case SPLT_TAGS_ARTIST:
-        return state->split.tags[index].artist;
-        break;
-      case SPLT_TAGS_ALBUM:
-        return state->split.tags[index].album;
-        break;
-      case SPLT_TAGS_YEAR:
-        return state->split.tags[index].year;
-        break;
-      case SPLT_TAGS_COMMENT:
-        return state->split.tags[index].comment;
-        break;
-      case SPLT_TAGS_PERFORMER:
-        return state->split.tags[index].performer;
-        break;
-      case SPLT_TAGS_TRACK:
-        return &state->split.tags[index].track;
-        break;
-      case SPLT_TAGS_VERSION:
-        return &state->split.tags[index].tags_version;
-        break;
-      case SPLT_TAGS_GENRE:
-        return &state->split.tags[index].genre;
-        break;
-      default:
-        splt_e_error(SPLT_IERROR_INT,__func__, index, NULL);
-        return NULL;
-    }
-  }
-
-  return NULL;
-}
-
-splt_tags *splt_tu_get_tags_like_x(splt_state *state)
-{
-  return &state->split.tags_like_x;
-}
-
-void splt_tu_free_tags(splt_state *state)
-{
-  if (state->split.tags)
-  {
-    int i = 0;
-    for (i = 0; i < state->split.real_tagsnumber; i++)
-    {
-      splt_tu_free_one_tags(&state->split.tags[i]);
-    }
-    free(state->split.tags);
-    state->split.tags = NULL;
-  }
-
-  state->split.real_tagsnumber = 0;
-
-  splt_tu_free_one_tags(splt_tu_get_tags_like_x(state));
-}
-
-splt_tags *splt_tu_get_current_tags(splt_state *state)
-{
-  int current_tags_number = splt_t_get_current_split_file_number(state) - 1;
-
-  int remaining_tags_like_x = splt_o_get_int_option(state, SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
-  if ((current_tags_number >= state->split.real_tagsnumber) &&
-      (remaining_tags_like_x != -1))
-  {
-    current_tags_number = remaining_tags_like_x; 
-  }
-
-  return splt_tu_get_tags_at(state, current_tags_number);
-}
-
-char *splt_tu_get_artist_or_performer_ptr(splt_tags *tags)
-{
-  if (!tags)
-  {
-    return NULL;
-  }
-
-  char *artist_or_performer = tags->artist;
-
-  if (tags->performer == NULL)
-  {
-    return artist_or_performer;
-  }
-
-  if (tags->performer[0] != '\0')
-  {
-    return tags->performer;
-  }
-
-  return artist_or_performer;
-}
-
-int splt_tu_copy_first_common_tags_on_all_tracks(splt_state *state, int tracks)
-{
-  int err = SPLT_OK;
-
-  char *artist0 = NULL;
-  char *album0 = NULL;
-  char *year0 = NULL;
-
-  char *first_artist = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_ARTIST);
-  err = splt_su_copy(first_artist, &artist0);
-  if (err < 0) { goto function_end; }
-
-  char *first_album = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_ALBUM);
-  err = splt_su_copy(first_album, &album0);
-  if (err < 0) { goto function_end; }
-
-  char *first_year = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_YEAR);
-  err = splt_su_copy(first_year, &year0);
-  if (err < 0) { goto function_end; }
-
-  unsigned char genre0 = 0x0;
-  unsigned char *g = (unsigned char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_GENRE);
-  if (g != NULL) {
-    genre0 = *g;
-  }
-
-  int i = 0;
-  for (i = 0; i < tracks;i++)
-  {
-    if (i != 0)
-    {
-      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_ARTIST, artist0);
-      if (err != SPLT_OK) { break; }
-
-      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_ALBUM, album0);
-      if (err != SPLT_OK) { break; }
-
-      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_YEAR, year0);
-      if (err != SPLT_OK) { break; }
-
-      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_GENRE, &genre0);
-      if (err != SPLT_OK) { break; }
-    }
-  }
-
-function_end:
-  if (artist0)
-  {
-    free(artist0);
-    artist0 = NULL;
-  }
-  if (album0)
-  {
-    free(album0);
-    album0 = NULL;
-  }
-  if (year0)
-  {
-    free(year0);
-    year0 = NULL;
-  }
-
-  return err;
-}
-
-static splt_tags *splt_tu_get_tags_to_replace_in_tags(splt_state *state)
-{
-  int current_tags_number = splt_t_get_current_split_file_number(state) - 1;
-
-  int remaining_tags_like_x = splt_o_get_int_option(state, SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
-  if ((current_tags_number >= state->split.real_tagsnumber) &&
-      (remaining_tags_like_x != -1))
-  {
-    return splt_tu_get_tags_like_x(state);
-  }
-
-  return splt_tu_get_tags_at(state, current_tags_number);
+  return state->original_tags.all_original_tags;
 }
 
 static char *splt_tu_get_replaced_with_tags(const char *word,
@@ -794,7 +584,20 @@ error:
   return NULL;
 }
 
-static void splt_tu_free_one_tags(splt_tags *tags)
+void splt_tu_free_one_tags(splt_tags **tags)
+{
+  if (!tags || !*tags)
+  {
+    return;
+  }
+
+  splt_tu_free_one_tags_content(*tags);
+
+  free(*tags);
+  *tags = NULL;
+}
+
+void splt_tu_free_one_tags_content(splt_tags *tags)
 {
   if (tags)
   {
@@ -828,16 +631,404 @@ static void splt_tu_free_one_tags(splt_tags *tags)
       free(tags->comment);
       tags->comment = NULL;
     }
+    if (tags->genre)
+    {
+      free(tags->genre);
+      tags->genre = NULL;
+    }
+
+    tags->track = -1;
   }
 }
 
-static void splt_tu_set_empty_tags(splt_state *state, int index)
+void splt_tu_append_tags_to_state(splt_state *state, splt_tags *tags, 
+    int append_new_tags, int *error)
 {
-  splt_tu_reset_tags(&state->split.tags[index]);
+  int err = SPLT_OK;
+
+  if (append_new_tags)
+  {
+    err = splt_tu_append_tags(state, tags->title, tags->artist, tags->album,
+        tags->performer, tags->year, tags->comment, tags->track, tags->genre,
+        SPLT_FALSE);
+  }
+  else
+  {
+    err = splt_tu_append_only_non_null_previous_tags(state, tags->title,
+        tags->artist, tags->album, tags->performer, tags->year, 
+        tags->comment, tags->track, tags->genre,
+        SPLT_TRUE);
+  }
+
+  if (err < 0)
+  {
+    *error = err;
+  }
 }
 
-static int splt_tu_set_on_tags_field(splt_tags *tags,
-    int tags_field, const void *data)
+void splt_tu_set_new_tags_where_current_tags_are_null(splt_state *state,
+    splt_tags *current_tags, splt_tags *new_tags,
+    int index, int *error)
+{
+  if (!current_tags->title)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_TITLE, new_tags->title);
+  }
+  if (!current_tags->artist)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_ARTIST, new_tags->artist);
+  }
+  if (!current_tags->album)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_ALBUM, new_tags->album);
+  }
+  if (!current_tags->performer)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_PERFORMER,
+        new_tags->performer);
+  }
+  if (!current_tags->year)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_YEAR, new_tags->year);
+  }
+  if (!current_tags->comment)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_COMMENT, new_tags->comment);
+  }
+  if (current_tags->track < 0)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_TRACK, &new_tags->track);
+  }
+  if (!current_tags->genre)
+  {
+    splt_tu_set_tags_field(state, index, SPLT_TAGS_GENRE, new_tags->genre);
+  }
+  splt_tu_set_tags_field(state, index, SPLT_TAGS_ORIGINAL, &new_tags->set_original_tags);
+}
+
+int splt_tu_has_one_tag_set(splt_tags *tags)
+{
+  if (tags->title != NULL ||
+      tags->artist != NULL ||
+      tags->album != NULL ||
+      tags->performer != NULL ||
+      tags->year != NULL ||
+      tags->comment != NULL ||
+      tags->track != -1 ||
+      tags->genre != NULL)
+  {
+    return SPLT_TRUE;
+  }
+
+  return SPLT_FALSE;
+}
+
+void splt_tu_copy_tags(splt_tags *from, splt_tags *to, int *error)
+{
+  int err = SPLT_OK;
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_TITLE, from->title);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_ARTIST, from->artist);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_ALBUM, from->album);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_YEAR, from->year);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_COMMENT, from->comment);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_PERFORMER, from->performer);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_TRACK, &from->track);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_ORIGINAL, &from->set_original_tags);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_GENRE, from->genre);
+  if (err < 0) { goto error; }
+
+  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_VERSION, &from->tags_version);
+  if (err < 0) { goto error; }
+
+  return;
+
+error:
+  *error = err;
+}
+
+static splt_tags *splt_tu_get_tags_to_replace_in_tags(splt_state *state)
+{
+  int current_tags_number = splt_t_get_current_split_file_number(state) - 1;
+
+  int remaining_tags_like_x = splt_o_get_int_option(state, SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
+
+  if ((current_tags_number >= state->split.real_tagsnumber) &&
+      (remaining_tags_like_x != -1))
+  {
+    return splt_tu_get_tags_like_x(state);
+  }
+
+  return splt_tu_get_tags_at(state, current_tags_number);
+}
+
+int splt_tu_set_tags_in_tags(splt_state *state, int current_split)
+{
+  int err = SPLT_OK;
+
+  splt_tags *tags = splt_tu_get_tags_to_replace_in_tags(state);
+  splt_tags *cur_tags = splt_tu_get_current_tags(state);
+
+  if (!tags || !cur_tags)
+  {
+    return err;
+  }
+
+  int track = -1;
+  if (tags->track > 0)
+  {
+    track = tags->track;
+  }
+  else if (splt_tu_has_one_tag_set(tags))
+  {
+    if (current_split != -1)
+    {
+      track = current_split + 1;
+    }
+    else
+    {
+      track = splt_t_get_current_split_file_number(state);
+    }
+  }
+
+  cur_tags->track = track;
+  cur_tags->tags_version = tags->tags_version;
+
+  int replace_tags_in_tags = splt_o_get_int_option(state, SPLT_OPT_REPLACE_TAGS_IN_TAGS);
+
+  char *t = splt_tu_get_replaced_with_tags(tags->title, tags, track, &err, replace_tags_in_tags);
+  if (err != SPLT_OK) { return err; }
+  char *y = splt_tu_get_replaced_with_tags(tags->year, tags, track, &err, replace_tags_in_tags);
+  if (err != SPLT_OK) { return err; }
+  char *a = splt_tu_get_replaced_with_tags(tags->artist, tags, track, &err, replace_tags_in_tags);
+  if (err != SPLT_OK) { return err; }
+  char *al = splt_tu_get_replaced_with_tags(tags->album, tags, track, &err, replace_tags_in_tags);
+  if (err != SPLT_OK) { return err; }
+  char *c = splt_tu_get_replaced_with_tags(tags->comment, tags, track, &err, replace_tags_in_tags);
+  if (err != SPLT_OK) { return err; }
+  char *g = splt_tu_get_replaced_with_tags(tags->genre, tags, track, &err, replace_tags_in_tags);
+  if (err != SPLT_OK) { return err; }
+
+  splt_su_free_replace(&cur_tags->title, t);
+  splt_su_free_replace(&cur_tags->year, y);
+  splt_su_free_replace(&cur_tags->artist, a);
+  splt_su_free_replace(&cur_tags->album, al);
+  splt_su_free_replace(&cur_tags->comment, c);
+  splt_su_free_replace(&cur_tags->genre, g);
+
+  return err;
+}
+
+splt_tags *splt_tu_get_tags_at(splt_state *state, int tags_index)
+{
+  if (!splt_tu_tags_exists(state, tags_index))
+  {
+    return NULL;
+  }
+
+  return &state->split.tags[tags_index];
+}
+
+splt_tags splt_tu_get_last_tags(splt_state *state)
+{
+  return state->split.tags[state->split.real_tagsnumber-1];
+}
+
+void *splt_tu_get_tags_field(splt_state *state, int index, int tags_field)
+{
+  if ((index >= state->split.real_tagsnumber) || (index < 0))
+  {
+    splt_e_error(SPLT_IERROR_INT,__func__, index, NULL);
+    return NULL;
+  }
+  else
+  {
+    switch(tags_field)
+    {
+      case SPLT_TAGS_TITLE:
+        return state->split.tags[index].title;
+        break;
+      case SPLT_TAGS_ARTIST:
+        return state->split.tags[index].artist;
+        break;
+      case SPLT_TAGS_ALBUM:
+        return state->split.tags[index].album;
+        break;
+      case SPLT_TAGS_YEAR:
+        return state->split.tags[index].year;
+        break;
+      case SPLT_TAGS_COMMENT:
+        return state->split.tags[index].comment;
+        break;
+      case SPLT_TAGS_PERFORMER:
+        return state->split.tags[index].performer;
+        break;
+      case SPLT_TAGS_TRACK:
+        return &state->split.tags[index].track;
+        break;
+      case SPLT_TAGS_VERSION:
+        return &state->split.tags[index].tags_version;
+        break;
+      case SPLT_TAGS_GENRE:
+        return state->split.tags[index].genre;
+        break;
+      case SPLT_TAGS_ORIGINAL:
+        return &state->split.tags[index].set_original_tags;
+        break;
+      default:
+        splt_e_error(SPLT_IERROR_INT,__func__, index, NULL);
+        return NULL;
+    }
+  }
+
+  return NULL;
+}
+
+splt_tags *splt_tu_get_tags_like_x(splt_state *state)
+{
+  return &state->split.tags_like_x;
+}
+
+void splt_tu_free_tags(splt_state *state)
+{
+  if (state->split.tags)
+  {
+    int i = 0;
+    for (i = 0; i < state->split.real_tagsnumber; i++)
+    {
+      splt_tu_free_one_tags_content(&state->split.tags[i]);
+    }
+    free(state->split.tags);
+    state->split.tags = NULL;
+  }
+
+  state->split.real_tagsnumber = 0;
+
+  splt_tu_free_one_tags_content(splt_tu_get_tags_like_x(state));
+}
+
+splt_tags *splt_tu_get_current_tags(splt_state *state)
+{
+  int current_tags_number = splt_t_get_current_split_file_number(state) - 1;
+
+  int remaining_tags_like_x = splt_o_get_int_option(state, SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
+  if ((current_tags_number >= state->split.real_tagsnumber) &&
+      (remaining_tags_like_x != -1))
+  {
+    current_tags_number = remaining_tags_like_x; 
+  }
+
+  return splt_tu_get_tags_at(state, current_tags_number);
+}
+
+char *splt_tu_get_artist_or_performer_ptr(splt_tags *tags)
+{
+  if (!tags)
+  {
+    return NULL;
+  }
+
+  char *artist_or_performer = tags->artist;
+
+  if (tags->performer == NULL)
+  {
+    return artist_or_performer;
+  }
+
+  if (tags->performer[0] != '\0')
+  {
+    return tags->performer;
+  }
+
+  return artist_or_performer;
+}
+
+int splt_tu_copy_first_common_tags_on_all_tracks(splt_state *state, int tracks)
+{
+  int err = SPLT_OK;
+
+  char *artist0 = NULL;
+  char *album0 = NULL;
+  char *year0 = NULL;
+  char *genre0 = NULL;
+
+  char *first_artist = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_ARTIST);
+  err = splt_su_copy(first_artist, &artist0);
+  if (err < 0) { goto function_end; }
+
+  char *first_album = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_ALBUM);
+  err = splt_su_copy(first_album, &album0);
+  if (err < 0) { goto function_end; }
+
+  char *first_year = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_YEAR);
+  err = splt_su_copy(first_year, &year0);
+  if (err < 0) { goto function_end; }
+
+  char *first_genre = (char *)splt_tu_get_tags_field(state, 0, SPLT_TAGS_GENRE);
+  err = splt_su_copy(first_genre, &genre0);
+  if (err < 0) { goto function_end; }
+
+  int i = 0;
+  for (i = 0; i < tracks;i++)
+  {
+    if (i != 0)
+    {
+      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_ARTIST, artist0);
+      if (err != SPLT_OK) { break; }
+
+      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_ALBUM, album0);
+      if (err != SPLT_OK) { break; }
+
+      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_YEAR, year0);
+      if (err != SPLT_OK) { break; }
+
+      err = splt_tu_set_tags_field(state, i, SPLT_TAGS_GENRE, genre0);
+      if (err != SPLT_OK) { break; }
+    }
+  }
+
+function_end:
+  if (artist0)
+  {
+    free(artist0);
+    artist0 = NULL;
+  }
+  if (album0)
+  {
+    free(album0);
+    album0 = NULL;
+  }
+  if (year0)
+  {
+    free(year0);
+    year0 = NULL;
+  }
+  if (genre0)
+  {
+    free(genre0);
+    genre0 = NULL;
+  }
+
+  return err;
+}
+
+int splt_tu_set_field_on_tags(splt_tags *tags, int tags_field, const void *data)
 {
   int err = SPLT_OK;
 
@@ -865,11 +1056,14 @@ static int splt_tu_set_on_tags_field(splt_tags *tags,
       tags->track = *((int *)data);
       break;
     case SPLT_TAGS_GENRE:
-      tags->genre = *((unsigned char *)data);
+      err = splt_su_copy((char *)data, &tags->genre);
       break;
     case SPLT_TAGS_VERSION:
       tags->tags_version = *((int *)data);
       break;
+    case SPLT_TAGS_ORIGINAL:
+      tags->set_original_tags = *((int *)data);
+      break;
     default:
       splt_e_error(SPLT_IERROR_INT,__func__, -500, NULL);
       break;
@@ -878,3 +1072,4 @@ static int splt_tu_set_on_tags_field(splt_tags *tags,
   return err;
 }
 
+
